﻿using EdFi.OdsApi.Sdk.Api.Resources;
using EdFi.OdsApi.Sdk.Client;
using EdFi.OdsApi.Sdk.Models.Resources;
using EdFi.OdsApi.SdkClient;


// Trust all SSL certs -- needed unless signed SSL certificates are configured.
System.Net.ServicePointManager.ServerCertificateValidationCallback =
    ((sender, certificate, chain, sslPolicyErrors) => true);

//Explicitly configures outgoing network calls to use the latest version of TLS where possible.
//Due to our reliance on some older libraries, the.NET framework won't necessarily default
//to the latest unless we explicitly request it. Some hosting environments will not allow older versions
//of TLS, and thus calls can fail without this extra configuration.
System.Net.ServicePointManager.SecurityProtocol |= System.Net.SecurityProtocolType.Tls13 | System.Net.SecurityProtocolType.Tls12;

// Oauth configuration
var oauthUrl = "https://uaapps.dpi.wi.gov/EdFiWebApiV6/";
var clientKey = "";
var clientSecret = "";

// TokenRetriever makes the oauth calls.  It has RestSharp dependency, install via NuGet
var tokenRetriever = new TokenRetriever(oauthUrl, clientKey, clientSecret);

// Plug Oauth access token. Tokens will need to be refreshed when they expire
var configuration = new Configuration() { AccessToken = tokenRetriever.ObtainNewBearerToken(), BasePath = "https://devapps.dpi.wi.gov/EdFiWebApiV6/data/v3/2024/" };

// GET students
var apiInstance = new SchoolsApi(configuration);
apiInstance.Configuration.DefaultHeaders.Add("Content-Type", "application/json");

// Fetch a single record with the totalCount flag set to true to retrieve the total number of records available
var schoolWithHttpInfo = apiInstance.GetSchoolsWithHttpInfo(limit: 1, offset: 0, totalCount: true);

var httpReponseCode = schoolWithHttpInfo.StatusCode; // returns System.Net.HttpStatusCode.OK
Console.WriteLine("Response code is " + httpReponseCode);

// Parse the total count value out of the "Total-Count" response header
int.TryParse(schoolWithHttpInfo.Headers["Total-Count"].First(), out var totalCount);

int offset = 0;
int limit = 100;
var schools = new List<EdFiSchool>();

while (offset < totalCount)
{
    Console.WriteLine($"Fetching school records {offset} through {Math.Min(offset + limit, totalCount)} of {totalCount}");
    schools.AddRange(apiInstance.GetSchools(limit: limit, offset: 0));
    offset += limit;
}

Console.WriteLine();
Console.WriteLine("Student Results");

foreach (var school in schools)
{
    Console.WriteLine($"School: {school.NameOfInstitution}, {school.Addresses.Select(a=>a.City).FirstOrDefault()}");
}

Console.WriteLine();
Console.WriteLine("Hit ENTER key to continue...");
Console.ReadLine();