/*
 * Identity API Endpoints
 *
 * The identity management API is used to access a native identity system if one exists in the enterprise. The endpoints provide client applications with the ability to find and (optionally) create unique person identifiers that can be used with Student, Staff, and Parent resources. The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 2
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Identities
{
    /// <summary>
    /// Enrollment
    /// </summary>
    [DataContract(Name = "Enrollment")]
    public partial class Enrollment : IEquatable<Enrollment>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Enrollment" /> class.
        /// </summary>
        /// <param name="schoolYear">schoolYear.</param>
        /// <param name="school">school.</param>
        /// <param name="entryGradeLevelDescriptor">entryGradeLevelDescriptor.</param>
        /// <param name="entryDate">entryDate.</param>
        /// <param name="exitWithdrawDate">exitWithdrawDate.</param>
        public Enrollment(int schoolYear = default(int), SchoolReference school = default(SchoolReference), string entryGradeLevelDescriptor = default(string), DateTime entryDate = default(DateTime), DateTime exitWithdrawDate = default(DateTime))
        {
            this.SchoolYear = schoolYear;
            this.School = school;
            this.EntryGradeLevelDescriptor = entryGradeLevelDescriptor;
            this.EntryDate = entryDate;
            this.ExitWithdrawDate = exitWithdrawDate;
        }

        /// <summary>
        /// Gets or Sets SchoolYear
        /// </summary>
        [DataMember(Name = "SchoolYear", EmitDefaultValue = false)]
        public int SchoolYear { get; set; }

        /// <summary>
        /// Gets or Sets School
        /// </summary>
        [DataMember(Name = "School", EmitDefaultValue = false)]
        public SchoolReference School { get; set; }

        /// <summary>
        /// Gets or Sets EntryGradeLevelDescriptor
        /// </summary>
        [DataMember(Name = "EntryGradeLevelDescriptor", EmitDefaultValue = false)]
        public string EntryGradeLevelDescriptor { get; set; }

        /// <summary>
        /// Gets or Sets EntryDate
        /// </summary>
        [DataMember(Name = "EntryDate", EmitDefaultValue = false)]
        public DateTime EntryDate { get; set; }

        /// <summary>
        /// Gets or Sets ExitWithdrawDate
        /// </summary>
        [DataMember(Name = "ExitWithdrawDate", EmitDefaultValue = false)]
        public DateTime ExitWithdrawDate { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class Enrollment {\n");
            sb.Append("  SchoolYear: ").Append(SchoolYear).Append("\n");
            sb.Append("  School: ").Append(School).Append("\n");
            sb.Append("  EntryGradeLevelDescriptor: ").Append(EntryGradeLevelDescriptor).Append("\n");
            sb.Append("  EntryDate: ").Append(EntryDate).Append("\n");
            sb.Append("  ExitWithdrawDate: ").Append(ExitWithdrawDate).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as Enrollment);
        }

        /// <summary>
        /// Returns true if Enrollment instances are equal
        /// </summary>
        /// <param name="input">Instance of Enrollment to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(Enrollment input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.SchoolYear == input.SchoolYear ||
                    this.SchoolYear.Equals(input.SchoolYear)
                ) && 
                (
                    this.School == input.School ||
                    (this.School != null &&
                    this.School.Equals(input.School))
                ) && 
                (
                    this.EntryGradeLevelDescriptor == input.EntryGradeLevelDescriptor ||
                    (this.EntryGradeLevelDescriptor != null &&
                    this.EntryGradeLevelDescriptor.Equals(input.EntryGradeLevelDescriptor))
                ) && 
                (
                    this.EntryDate == input.EntryDate ||
                    (this.EntryDate != null &&
                    this.EntryDate.Equals(input.EntryDate))
                ) && 
                (
                    this.ExitWithdrawDate == input.ExitWithdrawDate ||
                    (this.ExitWithdrawDate != null &&
                    this.ExitWithdrawDate.Equals(input.ExitWithdrawDate))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                hashCode = (hashCode * 59) + this.SchoolYear.GetHashCode();
                if (this.School != null)
                {
                    hashCode = (hashCode * 59) + this.School.GetHashCode();
                }
                if (this.EntryGradeLevelDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.EntryGradeLevelDescriptor.GetHashCode();
                }
                if (this.EntryDate != null)
                {
                    hashCode = (hashCode * 59) + this.EntryDate.GetHashCode();
                }
                if (this.ExitWithdrawDate != null)
                {
                    hashCode = (hashCode * 59) + this.ExitWithdrawDate.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
