/*
 * Identity API Endpoints
 *
 * The identity management API is used to access a native identity system if one exists in the enterprise. The endpoints provide client applications with the ability to find and (optionally) create unique person identifiers that can be used with Student, Staff, and Parent resources. The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 2
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Identities
{
    /// <summary>
    /// IdentitySearchResponse
    /// </summary>
    [DataContract(Name = "IdentitySearchResponse")]
    public partial class IdentitySearchResponse : IEquatable<IdentitySearchResponse>, IValidatableObject
    {
        /// <summary>
        /// Defines Status
        /// </summary>
        [JsonConverter(typeof(StringEnumConverter))]
        public enum StatusEnum
        {
            /// <summary>
            /// Enum Incomplete for value: Incomplete
            /// </summary>
            [EnumMember(Value = "Incomplete")]
            Incomplete = 1,

            /// <summary>
            /// Enum Complete for value: Complete
            /// </summary>
            [EnumMember(Value = "Complete")]
            Complete = 2

        }


        /// <summary>
        /// Gets or Sets Status
        /// </summary>
        [DataMember(Name = "Status", EmitDefaultValue = false)]
        public StatusEnum? Status { get; set; }
        /// <summary>
        /// Initializes a new instance of the <see cref="IdentitySearchResponse" /> class.
        /// </summary>
        /// <param name="searchResponses">searchResponses.</param>
        /// <param name="status">status.</param>
        /// <param name="searchKey">searchKey.</param>
        public IdentitySearchResponse(List<IdentitySearchResponses> searchResponses = default(List<IdentitySearchResponses>), StatusEnum? status = default(StatusEnum?), Guid searchKey = default(Guid))
        {
            this.SearchResponses = searchResponses;
            this.Status = status;
            this.SearchKey = searchKey;
        }

        /// <summary>
        /// Gets or Sets SearchResponses
        /// </summary>
        [DataMember(Name = "SearchResponses", EmitDefaultValue = false)]
        public List<IdentitySearchResponses> SearchResponses { get; set; }

        /// <summary>
        /// Gets or Sets SearchKey
        /// </summary>
        [DataMember(Name = "SearchKey", EmitDefaultValue = false)]
        public Guid SearchKey { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class IdentitySearchResponse {\n");
            sb.Append("  SearchResponses: ").Append(SearchResponses).Append("\n");
            sb.Append("  Status: ").Append(Status).Append("\n");
            sb.Append("  SearchKey: ").Append(SearchKey).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as IdentitySearchResponse);
        }

        /// <summary>
        /// Returns true if IdentitySearchResponse instances are equal
        /// </summary>
        /// <param name="input">Instance of IdentitySearchResponse to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(IdentitySearchResponse input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.SearchResponses == input.SearchResponses ||
                    this.SearchResponses != null &&
                    input.SearchResponses != null &&
                    this.SearchResponses.SequenceEqual(input.SearchResponses)
                ) && 
                (
                    this.Status == input.Status ||
                    this.Status.Equals(input.Status)
                ) && 
                (
                    this.SearchKey == input.SearchKey ||
                    (this.SearchKey != null &&
                    this.SearchKey.Equals(input.SearchKey))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.SearchResponses != null)
                {
                    hashCode = (hashCode * 59) + this.SearchResponses.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.Status.GetHashCode();
                if (this.SearchKey != null)
                {
                    hashCode = (hashCode * 59) + this.SearchKey.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
