/*
 * Identity API Endpoints
 *
 * The identity management API is used to access a native identity system if one exists in the enterprise. The endpoints provide client applications with the ability to find and (optionally) create unique person identifiers that can be used with Student, Staff, and Parent resources. The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 2
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Identities
{
    /// <summary>
    /// Local
    /// </summary>
    [DataContract(Name = "Local")]
    public partial class Local : IEquatable<Local>, IValidatableObject
    {
        /// <summary>
        /// Either Student or Teacher
        /// </summary>
        /// <value>Either Student or Teacher</value>
        [JsonConverter(typeof(StringEnumConverter))]
        public enum TypeDescriptionEnum
        {
            /// <summary>
            /// Enum Staff for value: Staff
            /// </summary>
            [EnumMember(Value = "Staff")]
            Staff = 1,

            /// <summary>
            /// Enum Student for value: Student
            /// </summary>
            [EnumMember(Value = "Student")]
            Student = 2

        }


        /// <summary>
        /// Either Student or Teacher
        /// </summary>
        /// <value>Either Student or Teacher</value>
        [DataMember(Name = "TypeDescription", EmitDefaultValue = false)]
        public TypeDescriptionEnum? TypeDescription { get; set; }
        /// <summary>
        /// Initializes a new instance of the <see cref="Local" /> class.
        /// </summary>
        /// <param name="educationOrganizationId">educationOrganizationId.</param>
        /// <param name="localPersonId">A string uniquely identifying the student for the Vendor Software.</param>
        /// <param name="description">A description of the person local row key - ex. Manually created wise id.</param>
        /// <param name="email">email.</param>
        /// <param name="typeDescription">Either Student or Teacher.</param>
        /// <param name="wiseSecureRoleDescription">A federation role Ex Student or Teacher.</param>
        public Local(int educationOrganizationId = default(int), string localPersonId = default(string), string description = default(string), string email = default(string), TypeDescriptionEnum? typeDescription = default(TypeDescriptionEnum?), string wiseSecureRoleDescription = default(string))
        {
            this.EducationOrganizationId = educationOrganizationId;
            this.LocalPersonId = localPersonId;
            this.Description = description;
            this.Email = email;
            this.TypeDescription = typeDescription;
            this.WiseSecureRoleDescription = wiseSecureRoleDescription;
        }

        /// <summary>
        /// Gets or Sets EducationOrganizationId
        /// </summary>
        [DataMember(Name = "EducationOrganizationId", EmitDefaultValue = false)]
        public int EducationOrganizationId { get; set; }

        /// <summary>
        /// A string uniquely identifying the student for the Vendor Software
        /// </summary>
        /// <value>A string uniquely identifying the student for the Vendor Software</value>
        [DataMember(Name = "LocalPersonId", EmitDefaultValue = false)]
        public string LocalPersonId { get; set; }

        /// <summary>
        /// A description of the person local row key - ex. Manually created wise id
        /// </summary>
        /// <value>A description of the person local row key - ex. Manually created wise id</value>
        [DataMember(Name = "Description", EmitDefaultValue = false)]
        public string Description { get; set; }

        /// <summary>
        /// Gets or Sets Email
        /// </summary>
        [DataMember(Name = "Email", EmitDefaultValue = false)]
        public string Email { get; set; }

        /// <summary>
        /// A federation role Ex Student or Teacher
        /// </summary>
        /// <value>A federation role Ex Student or Teacher</value>
        [DataMember(Name = "WiseSecureRoleDescription", EmitDefaultValue = false)]
        public string WiseSecureRoleDescription { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class Local {\n");
            sb.Append("  EducationOrganizationId: ").Append(EducationOrganizationId).Append("\n");
            sb.Append("  LocalPersonId: ").Append(LocalPersonId).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  Email: ").Append(Email).Append("\n");
            sb.Append("  TypeDescription: ").Append(TypeDescription).Append("\n");
            sb.Append("  WiseSecureRoleDescription: ").Append(WiseSecureRoleDescription).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as Local);
        }

        /// <summary>
        /// Returns true if Local instances are equal
        /// </summary>
        /// <param name="input">Instance of Local to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(Local input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.EducationOrganizationId == input.EducationOrganizationId ||
                    this.EducationOrganizationId.Equals(input.EducationOrganizationId)
                ) && 
                (
                    this.LocalPersonId == input.LocalPersonId ||
                    (this.LocalPersonId != null &&
                    this.LocalPersonId.Equals(input.LocalPersonId))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.Email == input.Email ||
                    (this.Email != null &&
                    this.Email.Equals(input.Email))
                ) && 
                (
                    this.TypeDescription == input.TypeDescription ||
                    this.TypeDescription.Equals(input.TypeDescription)
                ) && 
                (
                    this.WiseSecureRoleDescription == input.WiseSecureRoleDescription ||
                    (this.WiseSecureRoleDescription != null &&
                    this.WiseSecureRoleDescription.Equals(input.WiseSecureRoleDescription))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                hashCode = (hashCode * 59) + this.EducationOrganizationId.GetHashCode();
                if (this.LocalPersonId != null)
                {
                    hashCode = (hashCode * 59) + this.LocalPersonId.GetHashCode();
                }
                if (this.Description != null)
                {
                    hashCode = (hashCode * 59) + this.Description.GetHashCode();
                }
                if (this.Email != null)
                {
                    hashCode = (hashCode * 59) + this.Email.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.TypeDescription.GetHashCode();
                if (this.WiseSecureRoleDescription != null)
                {
                    hashCode = (hashCode * 59) + this.WiseSecureRoleDescription.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
