/*
 * Immunizations API
 *
 * The WI Department of Public Instruction in conjunction with the WI Department of Health Services is working to facilitate the retrieval of HL7 immunization data from the WI Immunization Registry via the EdFi API.
 *
 * The version of the OpenAPI document: v1
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Immunizations
{
    /// <summary>
    /// This data type transmits codes and the text associated with the code.
    /// </summary>
    [DataContract(Name = "ImmunizationsCodedElement")]
    public partial class ImmunizationsCodedElement : IEquatable<ImmunizationsCodedElement>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ImmunizationsCodedElement" /> class.
        /// </summary>
        /// <param name="identifier">Identifying code..</param>
        /// <param name="text">Human readable text that is not further used..</param>
        /// <param name="nameOfCodingSystem">Name of the coding system..</param>
        /// <param name="alternateIdentifier">Alternate identifying code..</param>
        /// <param name="alternateText">Alternate human readable text that is not further used..</param>
        /// <param name="alternateNameOfCodingSystem">Alternate name of the coding system..</param>
        public ImmunizationsCodedElement(string identifier = default(string), string text = default(string), string nameOfCodingSystem = default(string), string alternateIdentifier = default(string), string alternateText = default(string), string alternateNameOfCodingSystem = default(string))
        {
            this.Identifier = identifier;
            this.Text = text;
            this.NameOfCodingSystem = nameOfCodingSystem;
            this.AlternateIdentifier = alternateIdentifier;
            this.AlternateText = alternateText;
            this.AlternateNameOfCodingSystem = alternateNameOfCodingSystem;
        }

        /// <summary>
        /// Identifying code.
        /// </summary>
        /// <value>Identifying code.</value>
        [DataMember(Name = "identifier", EmitDefaultValue = false)]
        public string Identifier { get; set; }

        /// <summary>
        /// Human readable text that is not further used.
        /// </summary>
        /// <value>Human readable text that is not further used.</value>
        [DataMember(Name = "text", EmitDefaultValue = false)]
        public string Text { get; set; }

        /// <summary>
        /// Name of the coding system.
        /// </summary>
        /// <value>Name of the coding system.</value>
        [DataMember(Name = "nameOfCodingSystem", EmitDefaultValue = false)]
        public string NameOfCodingSystem { get; set; }

        /// <summary>
        /// Alternate identifying code.
        /// </summary>
        /// <value>Alternate identifying code.</value>
        [DataMember(Name = "alternateIdentifier", EmitDefaultValue = false)]
        public string AlternateIdentifier { get; set; }

        /// <summary>
        /// Alternate human readable text that is not further used.
        /// </summary>
        /// <value>Alternate human readable text that is not further used.</value>
        [DataMember(Name = "alternateText", EmitDefaultValue = false)]
        public string AlternateText { get; set; }

        /// <summary>
        /// Alternate name of the coding system.
        /// </summary>
        /// <value>Alternate name of the coding system.</value>
        [DataMember(Name = "alternateNameOfCodingSystem", EmitDefaultValue = false)]
        public string AlternateNameOfCodingSystem { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class ImmunizationsCodedElement {\n");
            sb.Append("  Identifier: ").Append(Identifier).Append("\n");
            sb.Append("  Text: ").Append(Text).Append("\n");
            sb.Append("  NameOfCodingSystem: ").Append(NameOfCodingSystem).Append("\n");
            sb.Append("  AlternateIdentifier: ").Append(AlternateIdentifier).Append("\n");
            sb.Append("  AlternateText: ").Append(AlternateText).Append("\n");
            sb.Append("  AlternateNameOfCodingSystem: ").Append(AlternateNameOfCodingSystem).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ImmunizationsCodedElement);
        }

        /// <summary>
        /// Returns true if ImmunizationsCodedElement instances are equal
        /// </summary>
        /// <param name="input">Instance of ImmunizationsCodedElement to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ImmunizationsCodedElement input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Identifier == input.Identifier ||
                    (this.Identifier != null &&
                    this.Identifier.Equals(input.Identifier))
                ) && 
                (
                    this.Text == input.Text ||
                    (this.Text != null &&
                    this.Text.Equals(input.Text))
                ) && 
                (
                    this.NameOfCodingSystem == input.NameOfCodingSystem ||
                    (this.NameOfCodingSystem != null &&
                    this.NameOfCodingSystem.Equals(input.NameOfCodingSystem))
                ) && 
                (
                    this.AlternateIdentifier == input.AlternateIdentifier ||
                    (this.AlternateIdentifier != null &&
                    this.AlternateIdentifier.Equals(input.AlternateIdentifier))
                ) && 
                (
                    this.AlternateText == input.AlternateText ||
                    (this.AlternateText != null &&
                    this.AlternateText.Equals(input.AlternateText))
                ) && 
                (
                    this.AlternateNameOfCodingSystem == input.AlternateNameOfCodingSystem ||
                    (this.AlternateNameOfCodingSystem != null &&
                    this.AlternateNameOfCodingSystem.Equals(input.AlternateNameOfCodingSystem))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Identifier != null)
                {
                    hashCode = (hashCode * 59) + this.Identifier.GetHashCode();
                }
                if (this.Text != null)
                {
                    hashCode = (hashCode * 59) + this.Text.GetHashCode();
                }
                if (this.NameOfCodingSystem != null)
                {
                    hashCode = (hashCode * 59) + this.NameOfCodingSystem.GetHashCode();
                }
                if (this.AlternateIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.AlternateIdentifier.GetHashCode();
                }
                if (this.AlternateText != null)
                {
                    hashCode = (hashCode * 59) + this.AlternateText.GetHashCode();
                }
                if (this.AlternateNameOfCodingSystem != null)
                {
                    hashCode = (hashCode * 59) + this.AlternateNameOfCodingSystem.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
