/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiChartOfAccount
    /// </summary>
    [DataContract(Name = "edFi_chartOfAccount")]
    public partial class EdFiChartOfAccount : IEquatable<EdFiChartOfAccount>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiChartOfAccount" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiChartOfAccount() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiChartOfAccount" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="accountIdentifier">SEA populated code value for the valid combination of account dimensions under which financials are reported. (required).</param>
        /// <param name="fiscalYear">The fiscal year for the account (required).</param>
        /// <param name="balanceSheetDimensionReference">balanceSheetDimensionReference.</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="functionDimensionReference">functionDimensionReference.</param>
        /// <param name="fundDimensionReference">fundDimensionReference.</param>
        /// <param name="objectDimensionReference">objectDimensionReference.</param>
        /// <param name="operationalUnitDimensionReference">operationalUnitDimensionReference.</param>
        /// <param name="programDimensionReference">programDimensionReference.</param>
        /// <param name="projectDimensionReference">projectDimensionReference.</param>
        /// <param name="sourceDimensionReference">sourceDimensionReference.</param>
        /// <param name="accountName">A descriptive name for the account..</param>
        /// <param name="accountTypeDescriptor">The type of account used in accounting such as revenue, expenditure, or balance sheet. (required).</param>
        /// <param name="reportingTags">An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiChartOfAccount(string id = default(string), string accountIdentifier = default(string), int fiscalYear = default(int), EdFiBalanceSheetDimensionReference balanceSheetDimensionReference = default(EdFiBalanceSheetDimensionReference), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), EdFiFunctionDimensionReference functionDimensionReference = default(EdFiFunctionDimensionReference), EdFiFundDimensionReference fundDimensionReference = default(EdFiFundDimensionReference), EdFiObjectDimensionReference objectDimensionReference = default(EdFiObjectDimensionReference), EdFiOperationalUnitDimensionReference operationalUnitDimensionReference = default(EdFiOperationalUnitDimensionReference), EdFiProgramDimensionReference programDimensionReference = default(EdFiProgramDimensionReference), EdFiProjectDimensionReference projectDimensionReference = default(EdFiProjectDimensionReference), EdFiSourceDimensionReference sourceDimensionReference = default(EdFiSourceDimensionReference), string accountName = default(string), string accountTypeDescriptor = default(string), List<EdFiChartOfAccountReportingTag> reportingTags = default(List<EdFiChartOfAccountReportingTag>), string etag = default(string))
        {
            // to ensure "accountIdentifier" is required (not null)
            if (accountIdentifier == null)
            {
                throw new ArgumentNullException("accountIdentifier is a required property for EdFiChartOfAccount and cannot be null");
            }
            this.AccountIdentifier = accountIdentifier;
            this.FiscalYear = fiscalYear;
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new ArgumentNullException("educationOrganizationReference is a required property for EdFiChartOfAccount and cannot be null");
            }
            this.EducationOrganizationReference = educationOrganizationReference;
            // to ensure "accountTypeDescriptor" is required (not null)
            if (accountTypeDescriptor == null)
            {
                throw new ArgumentNullException("accountTypeDescriptor is a required property for EdFiChartOfAccount and cannot be null");
            }
            this.AccountTypeDescriptor = accountTypeDescriptor;
            this.Id = id;
            this.BalanceSheetDimensionReference = balanceSheetDimensionReference;
            this.FunctionDimensionReference = functionDimensionReference;
            this.FundDimensionReference = fundDimensionReference;
            this.ObjectDimensionReference = objectDimensionReference;
            this.OperationalUnitDimensionReference = operationalUnitDimensionReference;
            this.ProgramDimensionReference = programDimensionReference;
            this.ProjectDimensionReference = projectDimensionReference;
            this.SourceDimensionReference = sourceDimensionReference;
            this.AccountName = accountName;
            this.ReportingTags = reportingTags;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// SEA populated code value for the valid combination of account dimensions under which financials are reported.
        /// </summary>
        /// <value>SEA populated code value for the valid combination of account dimensions under which financials are reported.</value>
        [DataMember(Name = "accountIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string AccountIdentifier { get; set; }

        /// <summary>
        /// The fiscal year for the account
        /// </summary>
        /// <value>The fiscal year for the account</value>
        [DataMember(Name = "fiscalYear", IsRequired = true, EmitDefaultValue = true)]
        public int FiscalYear { get; set; }

        /// <summary>
        /// Gets or Sets BalanceSheetDimensionReference
        /// </summary>
        [DataMember(Name = "balanceSheetDimensionReference", EmitDefaultValue = false)]
        public EdFiBalanceSheetDimensionReference BalanceSheetDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name = "educationOrganizationReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// Gets or Sets FunctionDimensionReference
        /// </summary>
        [DataMember(Name = "functionDimensionReference", EmitDefaultValue = false)]
        public EdFiFunctionDimensionReference FunctionDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets FundDimensionReference
        /// </summary>
        [DataMember(Name = "fundDimensionReference", EmitDefaultValue = false)]
        public EdFiFundDimensionReference FundDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ObjectDimensionReference
        /// </summary>
        [DataMember(Name = "objectDimensionReference", EmitDefaultValue = false)]
        public EdFiObjectDimensionReference ObjectDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets OperationalUnitDimensionReference
        /// </summary>
        [DataMember(Name = "operationalUnitDimensionReference", EmitDefaultValue = false)]
        public EdFiOperationalUnitDimensionReference OperationalUnitDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ProgramDimensionReference
        /// </summary>
        [DataMember(Name = "programDimensionReference", EmitDefaultValue = false)]
        public EdFiProgramDimensionReference ProgramDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets ProjectDimensionReference
        /// </summary>
        [DataMember(Name = "projectDimensionReference", EmitDefaultValue = false)]
        public EdFiProjectDimensionReference ProjectDimensionReference { get; set; }

        /// <summary>
        /// Gets or Sets SourceDimensionReference
        /// </summary>
        [DataMember(Name = "sourceDimensionReference", EmitDefaultValue = false)]
        public EdFiSourceDimensionReference SourceDimensionReference { get; set; }

        /// <summary>
        /// A descriptive name for the account.
        /// </summary>
        /// <value>A descriptive name for the account.</value>
        [DataMember(Name = "accountName", EmitDefaultValue = true)]
        public string AccountName { get; set; }

        /// <summary>
        /// The type of account used in accounting such as revenue, expenditure, or balance sheet.
        /// </summary>
        /// <value>The type of account used in accounting such as revenue, expenditure, or balance sheet.</value>
        [DataMember(Name = "accountTypeDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string AccountTypeDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting.
        /// </summary>
        /// <value>An unordered collection of chartOfAccountReportingTags. Optional tag for accountability reporting.</value>
        [DataMember(Name = "reportingTags", EmitDefaultValue = false)]
        public List<EdFiChartOfAccountReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiChartOfAccount {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AccountIdentifier: ").Append(AccountIdentifier).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  BalanceSheetDimensionReference: ").Append(BalanceSheetDimensionReference).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  FunctionDimensionReference: ").Append(FunctionDimensionReference).Append("\n");
            sb.Append("  FundDimensionReference: ").Append(FundDimensionReference).Append("\n");
            sb.Append("  ObjectDimensionReference: ").Append(ObjectDimensionReference).Append("\n");
            sb.Append("  OperationalUnitDimensionReference: ").Append(OperationalUnitDimensionReference).Append("\n");
            sb.Append("  ProgramDimensionReference: ").Append(ProgramDimensionReference).Append("\n");
            sb.Append("  ProjectDimensionReference: ").Append(ProjectDimensionReference).Append("\n");
            sb.Append("  SourceDimensionReference: ").Append(SourceDimensionReference).Append("\n");
            sb.Append("  AccountName: ").Append(AccountName).Append("\n");
            sb.Append("  AccountTypeDescriptor: ").Append(AccountTypeDescriptor).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiChartOfAccount);
        }

        /// <summary>
        /// Returns true if EdFiChartOfAccount instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiChartOfAccount to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiChartOfAccount input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AccountIdentifier == input.AccountIdentifier ||
                    (this.AccountIdentifier != null &&
                    this.AccountIdentifier.Equals(input.AccountIdentifier))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    this.FiscalYear.Equals(input.FiscalYear)
                ) && 
                (
                    this.BalanceSheetDimensionReference == input.BalanceSheetDimensionReference ||
                    (this.BalanceSheetDimensionReference != null &&
                    this.BalanceSheetDimensionReference.Equals(input.BalanceSheetDimensionReference))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.FunctionDimensionReference == input.FunctionDimensionReference ||
                    (this.FunctionDimensionReference != null &&
                    this.FunctionDimensionReference.Equals(input.FunctionDimensionReference))
                ) && 
                (
                    this.FundDimensionReference == input.FundDimensionReference ||
                    (this.FundDimensionReference != null &&
                    this.FundDimensionReference.Equals(input.FundDimensionReference))
                ) && 
                (
                    this.ObjectDimensionReference == input.ObjectDimensionReference ||
                    (this.ObjectDimensionReference != null &&
                    this.ObjectDimensionReference.Equals(input.ObjectDimensionReference))
                ) && 
                (
                    this.OperationalUnitDimensionReference == input.OperationalUnitDimensionReference ||
                    (this.OperationalUnitDimensionReference != null &&
                    this.OperationalUnitDimensionReference.Equals(input.OperationalUnitDimensionReference))
                ) && 
                (
                    this.ProgramDimensionReference == input.ProgramDimensionReference ||
                    (this.ProgramDimensionReference != null &&
                    this.ProgramDimensionReference.Equals(input.ProgramDimensionReference))
                ) && 
                (
                    this.ProjectDimensionReference == input.ProjectDimensionReference ||
                    (this.ProjectDimensionReference != null &&
                    this.ProjectDimensionReference.Equals(input.ProjectDimensionReference))
                ) && 
                (
                    this.SourceDimensionReference == input.SourceDimensionReference ||
                    (this.SourceDimensionReference != null &&
                    this.SourceDimensionReference.Equals(input.SourceDimensionReference))
                ) && 
                (
                    this.AccountName == input.AccountName ||
                    (this.AccountName != null &&
                    this.AccountName.Equals(input.AccountName))
                ) && 
                (
                    this.AccountTypeDescriptor == input.AccountTypeDescriptor ||
                    (this.AccountTypeDescriptor != null &&
                    this.AccountTypeDescriptor.Equals(input.AccountTypeDescriptor))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    input.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.AccountIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.AccountIdentifier.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.FiscalYear.GetHashCode();
                if (this.BalanceSheetDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.BalanceSheetDimensionReference.GetHashCode();
                }
                if (this.EducationOrganizationReference != null)
                {
                    hashCode = (hashCode * 59) + this.EducationOrganizationReference.GetHashCode();
                }
                if (this.FunctionDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.FunctionDimensionReference.GetHashCode();
                }
                if (this.FundDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.FundDimensionReference.GetHashCode();
                }
                if (this.ObjectDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.ObjectDimensionReference.GetHashCode();
                }
                if (this.OperationalUnitDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.OperationalUnitDimensionReference.GetHashCode();
                }
                if (this.ProgramDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.ProgramDimensionReference.GetHashCode();
                }
                if (this.ProjectDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.ProjectDimensionReference.GetHashCode();
                }
                if (this.SourceDimensionReference != null)
                {
                    hashCode = (hashCode * 59) + this.SourceDimensionReference.GetHashCode();
                }
                if (this.AccountName != null)
                {
                    hashCode = (hashCode * 59) + this.AccountName.GetHashCode();
                }
                if (this.AccountTypeDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.AccountTypeDescriptor.GetHashCode();
                }
                if (this.ReportingTags != null)
                {
                    hashCode = (hashCode * 59) + this.ReportingTags.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // AccountIdentifier (string) maxLength
            if (this.AccountIdentifier != null && this.AccountIdentifier.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountIdentifier, length must be less than 50.", new [] { "AccountIdentifier" });
            }

            // AccountName (string) maxLength
            if (this.AccountName != null && this.AccountName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountName, length must be less than 100.", new [] { "AccountName" });
            }

            // AccountTypeDescriptor (string) maxLength
            if (this.AccountTypeDescriptor != null && this.AccountTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountTypeDescriptor, length must be less than 306.", new [] { "AccountTypeDescriptor" });
            }

            yield break;
        }
    }

}
