/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiChartOfAccountReportingTag
    /// </summary>
    [DataContract(Name = "edFi_chartOfAccountReportingTag")]
    public partial class EdFiChartOfAccountReportingTag : IEquatable<EdFiChartOfAccountReportingTag>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiChartOfAccountReportingTag" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiChartOfAccountReportingTag() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiChartOfAccountReportingTag" /> class.
        /// </summary>
        /// <param name="reportingTagDescriptor">A descriptor used at the dimension and/or chart of account levels to demote specific state needs for reporting. (required).</param>
        /// <param name="tagValue">The value associated with the reporting tag..</param>
        public EdFiChartOfAccountReportingTag(string reportingTagDescriptor = default(string), string tagValue = default(string))
        {
            // to ensure "reportingTagDescriptor" is required (not null)
            if (reportingTagDescriptor == null)
            {
                throw new ArgumentNullException("reportingTagDescriptor is a required property for EdFiChartOfAccountReportingTag and cannot be null");
            }
            this.ReportingTagDescriptor = reportingTagDescriptor;
            this.TagValue = tagValue;
        }

        /// <summary>
        /// A descriptor used at the dimension and/or chart of account levels to demote specific state needs for reporting.
        /// </summary>
        /// <value>A descriptor used at the dimension and/or chart of account levels to demote specific state needs for reporting.</value>
        [DataMember(Name = "reportingTagDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ReportingTagDescriptor { get; set; }

        /// <summary>
        /// The value associated with the reporting tag.
        /// </summary>
        /// <value>The value associated with the reporting tag.</value>
        [DataMember(Name = "tagValue", EmitDefaultValue = true)]
        public string TagValue { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiChartOfAccountReportingTag {\n");
            sb.Append("  ReportingTagDescriptor: ").Append(ReportingTagDescriptor).Append("\n");
            sb.Append("  TagValue: ").Append(TagValue).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiChartOfAccountReportingTag);
        }

        /// <summary>
        /// Returns true if EdFiChartOfAccountReportingTag instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiChartOfAccountReportingTag to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiChartOfAccountReportingTag input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.ReportingTagDescriptor == input.ReportingTagDescriptor ||
                    (this.ReportingTagDescriptor != null &&
                    this.ReportingTagDescriptor.Equals(input.ReportingTagDescriptor))
                ) && 
                (
                    this.TagValue == input.TagValue ||
                    (this.TagValue != null &&
                    this.TagValue.Equals(input.TagValue))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.ReportingTagDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ReportingTagDescriptor.GetHashCode();
                }
                if (this.TagValue != null)
                {
                    hashCode = (hashCode * 59) + this.TagValue.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // ReportingTagDescriptor (string) maxLength
            if (this.ReportingTagDescriptor != null && this.ReportingTagDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ReportingTagDescriptor, length must be less than 306.", new [] { "ReportingTagDescriptor" });
            }

            // TagValue (string) maxLength
            if (this.TagValue != null && this.TagValue.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for TagValue, length must be less than 100.", new [] { "TagValue" });
            }

            yield break;
        }
    }

}
