/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiCommunityProvider
    /// </summary>
    [DataContract(Name = "edFi_communityProvider")]
    public partial class EdFiCommunityProvider : IEquatable<EdFiCommunityProvider>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCommunityProvider" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiCommunityProvider() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCommunityProvider" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="categories">An unordered collection of educationOrganizationCategories. The classification of the education agency within the geographic boundaries of a state according to the level of administrative and operational control granted by the state. (required).</param>
        /// <param name="communityProviderId">The identifier assigned to a community provider. (required).</param>
        /// <param name="communityOrganizationReference">communityOrganizationReference.</param>
        /// <param name="addresses">An unordered collection of educationOrganizationAddresses. The set of elements that describes an address for the education entity, including the street address, city, state, ZIP code, and ZIP code + 4..</param>
        /// <param name="identificationCodes">An unordered collection of educationOrganizationIdentificationCodes. A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity..</param>
        /// <param name="indicators">An unordered collection of educationOrganizationIndicators. An indicator or metric of an education organization..</param>
        /// <param name="institutionTelephones">An unordered collection of educationOrganizationInstitutionTelephones. The 10-digit telephone number, including the area code, for the education entity..</param>
        /// <param name="internationalAddresses">An unordered collection of educationOrganizationInternationalAddresses. The set of elements that describes the international physical location of the education entity..</param>
        /// <param name="licenseExemptIndicator">An indication of whether the provider is exempt from having a license..</param>
        /// <param name="nameOfInstitution">The full, legally accepted name of the institution. (required).</param>
        /// <param name="operationalStatusDescriptor">The current operational status of the education organization (e.g., active, inactive)..</param>
        /// <param name="providerCategoryDescriptor">Indicates the category of the provider. (required).</param>
        /// <param name="providerProfitabilityDescriptor">Indicates the profitability status of the provider..</param>
        /// <param name="providerStatusDescriptor">Indicates the status of the provider. (required).</param>
        /// <param name="schoolIndicator">An indication of whether the community provider is a school..</param>
        /// <param name="shortNameOfInstitution">A short name for the institution..</param>
        /// <param name="webSite">The public web site address (URL) for the education organization..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiCommunityProvider(string id = default(string), List<EdFiEducationOrganizationCategory> categories = default(List<EdFiEducationOrganizationCategory>), int communityProviderId = default(int), EdFiCommunityOrganizationReference communityOrganizationReference = default(EdFiCommunityOrganizationReference), List<EdFiEducationOrganizationAddress> addresses = default(List<EdFiEducationOrganizationAddress>), List<EdFiEducationOrganizationIdentificationCode> identificationCodes = default(List<EdFiEducationOrganizationIdentificationCode>), List<EdFiEducationOrganizationIndicator> indicators = default(List<EdFiEducationOrganizationIndicator>), List<EdFiEducationOrganizationInstitutionTelephone> institutionTelephones = default(List<EdFiEducationOrganizationInstitutionTelephone>), List<EdFiEducationOrganizationInternationalAddress> internationalAddresses = default(List<EdFiEducationOrganizationInternationalAddress>), bool? licenseExemptIndicator = default(bool?), string nameOfInstitution = default(string), string operationalStatusDescriptor = default(string), string providerCategoryDescriptor = default(string), string providerProfitabilityDescriptor = default(string), string providerStatusDescriptor = default(string), bool? schoolIndicator = default(bool?), string shortNameOfInstitution = default(string), string webSite = default(string), string etag = default(string))
        {
            // to ensure "categories" is required (not null)
            if (categories == null)
            {
                throw new ArgumentNullException("categories is a required property for EdFiCommunityProvider and cannot be null");
            }
            this.Categories = categories;
            this.CommunityProviderId = communityProviderId;
            // to ensure "nameOfInstitution" is required (not null)
            if (nameOfInstitution == null)
            {
                throw new ArgumentNullException("nameOfInstitution is a required property for EdFiCommunityProvider and cannot be null");
            }
            this.NameOfInstitution = nameOfInstitution;
            // to ensure "providerCategoryDescriptor" is required (not null)
            if (providerCategoryDescriptor == null)
            {
                throw new ArgumentNullException("providerCategoryDescriptor is a required property for EdFiCommunityProvider and cannot be null");
            }
            this.ProviderCategoryDescriptor = providerCategoryDescriptor;
            // to ensure "providerStatusDescriptor" is required (not null)
            if (providerStatusDescriptor == null)
            {
                throw new ArgumentNullException("providerStatusDescriptor is a required property for EdFiCommunityProvider and cannot be null");
            }
            this.ProviderStatusDescriptor = providerStatusDescriptor;
            this.Id = id;
            this.CommunityOrganizationReference = communityOrganizationReference;
            this.Addresses = addresses;
            this.IdentificationCodes = identificationCodes;
            this.Indicators = indicators;
            this.InstitutionTelephones = institutionTelephones;
            this.InternationalAddresses = internationalAddresses;
            this.LicenseExemptIndicator = licenseExemptIndicator;
            this.OperationalStatusDescriptor = operationalStatusDescriptor;
            this.ProviderProfitabilityDescriptor = providerProfitabilityDescriptor;
            this.SchoolIndicator = schoolIndicator;
            this.ShortNameOfInstitution = shortNameOfInstitution;
            this.WebSite = webSite;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationCategories. The classification of the education agency within the geographic boundaries of a state according to the level of administrative and operational control granted by the state.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationCategories. The classification of the education agency within the geographic boundaries of a state according to the level of administrative and operational control granted by the state.</value>
        [DataMember(Name = "categories", IsRequired = true, EmitDefaultValue = true)]
        public List<EdFiEducationOrganizationCategory> Categories { get; set; }

        /// <summary>
        /// The identifier assigned to a community provider.
        /// </summary>
        /// <value>The identifier assigned to a community provider.</value>
        [DataMember(Name = "communityProviderId", IsRequired = true, EmitDefaultValue = true)]
        public int CommunityProviderId { get; set; }

        /// <summary>
        /// Gets or Sets CommunityOrganizationReference
        /// </summary>
        [DataMember(Name = "communityOrganizationReference", EmitDefaultValue = false)]
        public EdFiCommunityOrganizationReference CommunityOrganizationReference { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationAddresses. The set of elements that describes an address for the education entity, including the street address, city, state, ZIP code, and ZIP code + 4.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationAddresses. The set of elements that describes an address for the education entity, including the street address, city, state, ZIP code, and ZIP code + 4.</value>
        [DataMember(Name = "addresses", EmitDefaultValue = false)]
        public List<EdFiEducationOrganizationAddress> Addresses { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationIdentificationCodes. A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationIdentificationCodes. A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</value>
        [DataMember(Name = "identificationCodes", EmitDefaultValue = false)]
        public List<EdFiEducationOrganizationIdentificationCode> IdentificationCodes { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationIndicators. An indicator or metric of an education organization.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationIndicators. An indicator or metric of an education organization.</value>
        [DataMember(Name = "indicators", EmitDefaultValue = false)]
        public List<EdFiEducationOrganizationIndicator> Indicators { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationInstitutionTelephones. The 10-digit telephone number, including the area code, for the education entity.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationInstitutionTelephones. The 10-digit telephone number, including the area code, for the education entity.</value>
        [DataMember(Name = "institutionTelephones", EmitDefaultValue = false)]
        public List<EdFiEducationOrganizationInstitutionTelephone> InstitutionTelephones { get; set; }

        /// <summary>
        /// An unordered collection of educationOrganizationInternationalAddresses. The set of elements that describes the international physical location of the education entity.
        /// </summary>
        /// <value>An unordered collection of educationOrganizationInternationalAddresses. The set of elements that describes the international physical location of the education entity.</value>
        [DataMember(Name = "internationalAddresses", EmitDefaultValue = false)]
        public List<EdFiEducationOrganizationInternationalAddress> InternationalAddresses { get; set; }

        /// <summary>
        /// An indication of whether the provider is exempt from having a license.
        /// </summary>
        /// <value>An indication of whether the provider is exempt from having a license.</value>
        [DataMember(Name = "licenseExemptIndicator", EmitDefaultValue = true)]
        public bool? LicenseExemptIndicator { get; set; }

        /// <summary>
        /// The full, legally accepted name of the institution.
        /// </summary>
        /// <value>The full, legally accepted name of the institution.</value>
        [DataMember(Name = "nameOfInstitution", IsRequired = true, EmitDefaultValue = true)]
        public string NameOfInstitution { get; set; }

        /// <summary>
        /// The current operational status of the education organization (e.g., active, inactive).
        /// </summary>
        /// <value>The current operational status of the education organization (e.g., active, inactive).</value>
        [DataMember(Name = "operationalStatusDescriptor", EmitDefaultValue = true)]
        public string OperationalStatusDescriptor { get; set; }

        /// <summary>
        /// Indicates the category of the provider.
        /// </summary>
        /// <value>Indicates the category of the provider.</value>
        [DataMember(Name = "providerCategoryDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ProviderCategoryDescriptor { get; set; }

        /// <summary>
        /// Indicates the profitability status of the provider.
        /// </summary>
        /// <value>Indicates the profitability status of the provider.</value>
        [DataMember(Name = "providerProfitabilityDescriptor", EmitDefaultValue = true)]
        public string ProviderProfitabilityDescriptor { get; set; }

        /// <summary>
        /// Indicates the status of the provider.
        /// </summary>
        /// <value>Indicates the status of the provider.</value>
        [DataMember(Name = "providerStatusDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ProviderStatusDescriptor { get; set; }

        /// <summary>
        /// An indication of whether the community provider is a school.
        /// </summary>
        /// <value>An indication of whether the community provider is a school.</value>
        [DataMember(Name = "schoolIndicator", EmitDefaultValue = true)]
        public bool? SchoolIndicator { get; set; }

        /// <summary>
        /// A short name for the institution.
        /// </summary>
        /// <value>A short name for the institution.</value>
        [DataMember(Name = "shortNameOfInstitution", EmitDefaultValue = true)]
        public string ShortNameOfInstitution { get; set; }

        /// <summary>
        /// The public web site address (URL) for the education organization.
        /// </summary>
        /// <value>The public web site address (URL) for the education organization.</value>
        [DataMember(Name = "webSite", EmitDefaultValue = true)]
        public string WebSite { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiCommunityProvider {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  Categories: ").Append(Categories).Append("\n");
            sb.Append("  CommunityProviderId: ").Append(CommunityProviderId).Append("\n");
            sb.Append("  CommunityOrganizationReference: ").Append(CommunityOrganizationReference).Append("\n");
            sb.Append("  Addresses: ").Append(Addresses).Append("\n");
            sb.Append("  IdentificationCodes: ").Append(IdentificationCodes).Append("\n");
            sb.Append("  Indicators: ").Append(Indicators).Append("\n");
            sb.Append("  InstitutionTelephones: ").Append(InstitutionTelephones).Append("\n");
            sb.Append("  InternationalAddresses: ").Append(InternationalAddresses).Append("\n");
            sb.Append("  LicenseExemptIndicator: ").Append(LicenseExemptIndicator).Append("\n");
            sb.Append("  NameOfInstitution: ").Append(NameOfInstitution).Append("\n");
            sb.Append("  OperationalStatusDescriptor: ").Append(OperationalStatusDescriptor).Append("\n");
            sb.Append("  ProviderCategoryDescriptor: ").Append(ProviderCategoryDescriptor).Append("\n");
            sb.Append("  ProviderProfitabilityDescriptor: ").Append(ProviderProfitabilityDescriptor).Append("\n");
            sb.Append("  ProviderStatusDescriptor: ").Append(ProviderStatusDescriptor).Append("\n");
            sb.Append("  SchoolIndicator: ").Append(SchoolIndicator).Append("\n");
            sb.Append("  ShortNameOfInstitution: ").Append(ShortNameOfInstitution).Append("\n");
            sb.Append("  WebSite: ").Append(WebSite).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiCommunityProvider);
        }

        /// <summary>
        /// Returns true if EdFiCommunityProvider instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiCommunityProvider to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiCommunityProvider input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.Categories == input.Categories ||
                    this.Categories != null &&
                    input.Categories != null &&
                    this.Categories.SequenceEqual(input.Categories)
                ) && 
                (
                    this.CommunityProviderId == input.CommunityProviderId ||
                    this.CommunityProviderId.Equals(input.CommunityProviderId)
                ) && 
                (
                    this.CommunityOrganizationReference == input.CommunityOrganizationReference ||
                    (this.CommunityOrganizationReference != null &&
                    this.CommunityOrganizationReference.Equals(input.CommunityOrganizationReference))
                ) && 
                (
                    this.Addresses == input.Addresses ||
                    this.Addresses != null &&
                    input.Addresses != null &&
                    this.Addresses.SequenceEqual(input.Addresses)
                ) && 
                (
                    this.IdentificationCodes == input.IdentificationCodes ||
                    this.IdentificationCodes != null &&
                    input.IdentificationCodes != null &&
                    this.IdentificationCodes.SequenceEqual(input.IdentificationCodes)
                ) && 
                (
                    this.Indicators == input.Indicators ||
                    this.Indicators != null &&
                    input.Indicators != null &&
                    this.Indicators.SequenceEqual(input.Indicators)
                ) && 
                (
                    this.InstitutionTelephones == input.InstitutionTelephones ||
                    this.InstitutionTelephones != null &&
                    input.InstitutionTelephones != null &&
                    this.InstitutionTelephones.SequenceEqual(input.InstitutionTelephones)
                ) && 
                (
                    this.InternationalAddresses == input.InternationalAddresses ||
                    this.InternationalAddresses != null &&
                    input.InternationalAddresses != null &&
                    this.InternationalAddresses.SequenceEqual(input.InternationalAddresses)
                ) && 
                (
                    this.LicenseExemptIndicator == input.LicenseExemptIndicator ||
                    (this.LicenseExemptIndicator != null &&
                    this.LicenseExemptIndicator.Equals(input.LicenseExemptIndicator))
                ) && 
                (
                    this.NameOfInstitution == input.NameOfInstitution ||
                    (this.NameOfInstitution != null &&
                    this.NameOfInstitution.Equals(input.NameOfInstitution))
                ) && 
                (
                    this.OperationalStatusDescriptor == input.OperationalStatusDescriptor ||
                    (this.OperationalStatusDescriptor != null &&
                    this.OperationalStatusDescriptor.Equals(input.OperationalStatusDescriptor))
                ) && 
                (
                    this.ProviderCategoryDescriptor == input.ProviderCategoryDescriptor ||
                    (this.ProviderCategoryDescriptor != null &&
                    this.ProviderCategoryDescriptor.Equals(input.ProviderCategoryDescriptor))
                ) && 
                (
                    this.ProviderProfitabilityDescriptor == input.ProviderProfitabilityDescriptor ||
                    (this.ProviderProfitabilityDescriptor != null &&
                    this.ProviderProfitabilityDescriptor.Equals(input.ProviderProfitabilityDescriptor))
                ) && 
                (
                    this.ProviderStatusDescriptor == input.ProviderStatusDescriptor ||
                    (this.ProviderStatusDescriptor != null &&
                    this.ProviderStatusDescriptor.Equals(input.ProviderStatusDescriptor))
                ) && 
                (
                    this.SchoolIndicator == input.SchoolIndicator ||
                    (this.SchoolIndicator != null &&
                    this.SchoolIndicator.Equals(input.SchoolIndicator))
                ) && 
                (
                    this.ShortNameOfInstitution == input.ShortNameOfInstitution ||
                    (this.ShortNameOfInstitution != null &&
                    this.ShortNameOfInstitution.Equals(input.ShortNameOfInstitution))
                ) && 
                (
                    this.WebSite == input.WebSite ||
                    (this.WebSite != null &&
                    this.WebSite.Equals(input.WebSite))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.Categories != null)
                {
                    hashCode = (hashCode * 59) + this.Categories.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.CommunityProviderId.GetHashCode();
                if (this.CommunityOrganizationReference != null)
                {
                    hashCode = (hashCode * 59) + this.CommunityOrganizationReference.GetHashCode();
                }
                if (this.Addresses != null)
                {
                    hashCode = (hashCode * 59) + this.Addresses.GetHashCode();
                }
                if (this.IdentificationCodes != null)
                {
                    hashCode = (hashCode * 59) + this.IdentificationCodes.GetHashCode();
                }
                if (this.Indicators != null)
                {
                    hashCode = (hashCode * 59) + this.Indicators.GetHashCode();
                }
                if (this.InstitutionTelephones != null)
                {
                    hashCode = (hashCode * 59) + this.InstitutionTelephones.GetHashCode();
                }
                if (this.InternationalAddresses != null)
                {
                    hashCode = (hashCode * 59) + this.InternationalAddresses.GetHashCode();
                }
                if (this.LicenseExemptIndicator != null)
                {
                    hashCode = (hashCode * 59) + this.LicenseExemptIndicator.GetHashCode();
                }
                if (this.NameOfInstitution != null)
                {
                    hashCode = (hashCode * 59) + this.NameOfInstitution.GetHashCode();
                }
                if (this.OperationalStatusDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.OperationalStatusDescriptor.GetHashCode();
                }
                if (this.ProviderCategoryDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ProviderCategoryDescriptor.GetHashCode();
                }
                if (this.ProviderProfitabilityDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ProviderProfitabilityDescriptor.GetHashCode();
                }
                if (this.ProviderStatusDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ProviderStatusDescriptor.GetHashCode();
                }
                if (this.SchoolIndicator != null)
                {
                    hashCode = (hashCode * 59) + this.SchoolIndicator.GetHashCode();
                }
                if (this.ShortNameOfInstitution != null)
                {
                    hashCode = (hashCode * 59) + this.ShortNameOfInstitution.GetHashCode();
                }
                if (this.WebSite != null)
                {
                    hashCode = (hashCode * 59) + this.WebSite.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // NameOfInstitution (string) maxLength
            if (this.NameOfInstitution != null && this.NameOfInstitution.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for NameOfInstitution, length must be less than 75.", new [] { "NameOfInstitution" });
            }

            // OperationalStatusDescriptor (string) maxLength
            if (this.OperationalStatusDescriptor != null && this.OperationalStatusDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for OperationalStatusDescriptor, length must be less than 306.", new [] { "OperationalStatusDescriptor" });
            }

            // ProviderCategoryDescriptor (string) maxLength
            if (this.ProviderCategoryDescriptor != null && this.ProviderCategoryDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProviderCategoryDescriptor, length must be less than 306.", new [] { "ProviderCategoryDescriptor" });
            }

            // ProviderProfitabilityDescriptor (string) maxLength
            if (this.ProviderProfitabilityDescriptor != null && this.ProviderProfitabilityDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProviderProfitabilityDescriptor, length must be less than 306.", new [] { "ProviderProfitabilityDescriptor" });
            }

            // ProviderStatusDescriptor (string) maxLength
            if (this.ProviderStatusDescriptor != null && this.ProviderStatusDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProviderStatusDescriptor, length must be less than 306.", new [] { "ProviderStatusDescriptor" });
            }

            // ShortNameOfInstitution (string) maxLength
            if (this.ShortNameOfInstitution != null && this.ShortNameOfInstitution.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ShortNameOfInstitution, length must be less than 75.", new [] { "ShortNameOfInstitution" });
            }

            // WebSite (string) maxLength
            if (this.WebSite != null && this.WebSite.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for WebSite, length must be less than 255.", new [] { "WebSite" });
            }

            yield break;
        }
    }

}
