/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiCompetencyObjective
    /// </summary>
    [DataContract(Name = "edFi_competencyObjective")]
    public partial class EdFiCompetencyObjective : IEquatable<EdFiCompetencyObjective>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCompetencyObjective" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiCompetencyObjective() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiCompetencyObjective" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="objective">The designated title of the competency objective. (required).</param>
        /// <param name="objectiveGradeLevelDescriptor">The grade level for which the competency objective is targeted. (required).</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="competencyObjectiveId">The Identifier for the competency objective..</param>
        /// <param name="description">The description of the student competency objective..</param>
        /// <param name="successCriteria">One or more statements that describes the criteria used by teachers and students to check for attainment of a competency objective. This criteria gives clear indications as to the degree to which learning is moving through the Zone or Proximal Development toward independent achievement of the competency objective..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiCompetencyObjective(string id = default(string), string objective = default(string), string objectiveGradeLevelDescriptor = default(string), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), string competencyObjectiveId = default(string), string description = default(string), string successCriteria = default(string), string etag = default(string))
        {
            // to ensure "objective" is required (not null)
            if (objective == null)
            {
                throw new ArgumentNullException("objective is a required property for EdFiCompetencyObjective and cannot be null");
            }
            this.Objective = objective;
            // to ensure "objectiveGradeLevelDescriptor" is required (not null)
            if (objectiveGradeLevelDescriptor == null)
            {
                throw new ArgumentNullException("objectiveGradeLevelDescriptor is a required property for EdFiCompetencyObjective and cannot be null");
            }
            this.ObjectiveGradeLevelDescriptor = objectiveGradeLevelDescriptor;
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new ArgumentNullException("educationOrganizationReference is a required property for EdFiCompetencyObjective and cannot be null");
            }
            this.EducationOrganizationReference = educationOrganizationReference;
            this.Id = id;
            this.CompetencyObjectiveId = competencyObjectiveId;
            this.Description = description;
            this.SuccessCriteria = successCriteria;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// The designated title of the competency objective.
        /// </summary>
        /// <value>The designated title of the competency objective.</value>
        [DataMember(Name = "objective", IsRequired = true, EmitDefaultValue = true)]
        public string Objective { get; set; }

        /// <summary>
        /// The grade level for which the competency objective is targeted.
        /// </summary>
        /// <value>The grade level for which the competency objective is targeted.</value>
        [DataMember(Name = "objectiveGradeLevelDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ObjectiveGradeLevelDescriptor { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name = "educationOrganizationReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// The Identifier for the competency objective.
        /// </summary>
        /// <value>The Identifier for the competency objective.</value>
        [DataMember(Name = "competencyObjectiveId", EmitDefaultValue = true)]
        public string CompetencyObjectiveId { get; set; }

        /// <summary>
        /// The description of the student competency objective.
        /// </summary>
        /// <value>The description of the student competency objective.</value>
        [DataMember(Name = "description", EmitDefaultValue = true)]
        public string Description { get; set; }

        /// <summary>
        /// One or more statements that describes the criteria used by teachers and students to check for attainment of a competency objective. This criteria gives clear indications as to the degree to which learning is moving through the Zone or Proximal Development toward independent achievement of the competency objective.
        /// </summary>
        /// <value>One or more statements that describes the criteria used by teachers and students to check for attainment of a competency objective. This criteria gives clear indications as to the degree to which learning is moving through the Zone or Proximal Development toward independent achievement of the competency objective.</value>
        [DataMember(Name = "successCriteria", EmitDefaultValue = true)]
        public string SuccessCriteria { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiCompetencyObjective {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  Objective: ").Append(Objective).Append("\n");
            sb.Append("  ObjectiveGradeLevelDescriptor: ").Append(ObjectiveGradeLevelDescriptor).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  CompetencyObjectiveId: ").Append(CompetencyObjectiveId).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  SuccessCriteria: ").Append(SuccessCriteria).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiCompetencyObjective);
        }

        /// <summary>
        /// Returns true if EdFiCompetencyObjective instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiCompetencyObjective to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiCompetencyObjective input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.Objective == input.Objective ||
                    (this.Objective != null &&
                    this.Objective.Equals(input.Objective))
                ) && 
                (
                    this.ObjectiveGradeLevelDescriptor == input.ObjectiveGradeLevelDescriptor ||
                    (this.ObjectiveGradeLevelDescriptor != null &&
                    this.ObjectiveGradeLevelDescriptor.Equals(input.ObjectiveGradeLevelDescriptor))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.CompetencyObjectiveId == input.CompetencyObjectiveId ||
                    (this.CompetencyObjectiveId != null &&
                    this.CompetencyObjectiveId.Equals(input.CompetencyObjectiveId))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.SuccessCriteria == input.SuccessCriteria ||
                    (this.SuccessCriteria != null &&
                    this.SuccessCriteria.Equals(input.SuccessCriteria))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.Objective != null)
                {
                    hashCode = (hashCode * 59) + this.Objective.GetHashCode();
                }
                if (this.ObjectiveGradeLevelDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ObjectiveGradeLevelDescriptor.GetHashCode();
                }
                if (this.EducationOrganizationReference != null)
                {
                    hashCode = (hashCode * 59) + this.EducationOrganizationReference.GetHashCode();
                }
                if (this.CompetencyObjectiveId != null)
                {
                    hashCode = (hashCode * 59) + this.CompetencyObjectiveId.GetHashCode();
                }
                if (this.Description != null)
                {
                    hashCode = (hashCode * 59) + this.Description.GetHashCode();
                }
                if (this.SuccessCriteria != null)
                {
                    hashCode = (hashCode * 59) + this.SuccessCriteria.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // Objective (string) maxLength
            if (this.Objective != null && this.Objective.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Objective, length must be less than 60.", new [] { "Objective" });
            }

            // ObjectiveGradeLevelDescriptor (string) maxLength
            if (this.ObjectiveGradeLevelDescriptor != null && this.ObjectiveGradeLevelDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ObjectiveGradeLevelDescriptor, length must be less than 306.", new [] { "ObjectiveGradeLevelDescriptor" });
            }

            // CompetencyObjectiveId (string) maxLength
            if (this.CompetencyObjectiveId != null && this.CompetencyObjectiveId.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CompetencyObjectiveId, length must be less than 60.", new [] { "CompetencyObjectiveId" });
            }

            // Description (string) maxLength
            if (this.Description != null && this.Description.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Description, length must be less than 1024.", new [] { "Description" });
            }

            // SuccessCriteria (string) maxLength
            if (this.SuccessCriteria != null && this.SuccessCriteria.Length > 150)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SuccessCriteria, length must be less than 150.", new [] { "SuccessCriteria" });
            }

            yield break;
        }
    }

}
