/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiDisciplineIncident
    /// </summary>
    [DataContract(Name = "edFi_disciplineIncident")]
    public partial class EdFiDisciplineIncident : IEquatable<EdFiDisciplineIncident>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncident" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiDisciplineIncident() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncident" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="incidentIdentifier">A locally assigned unique identifier (within the school or school district) to identify each specific DisciplineIncident or occurrence. The same identifier should be used to document the entire discipline incident even if it included multiple offenses and multiple offenders. (required).</param>
        /// <param name="schoolReference">schoolReference (required).</param>
        /// <param name="staffReference">staffReference.</param>
        /// <param name="behaviors">An unordered collection of disciplineIncidentBehaviors. Describes behavior by category and provides a detailed description..</param>
        /// <param name="caseNumber">The case number assigned to the DisciplineIncident by law enforcement or other organization..</param>
        /// <param name="externalParticipants">An unordered collection of disciplineIncidentExternalParticipants. Information on an individual involved in the discipline incident..</param>
        /// <param name="incidentCost">The value of any quantifiable monetary loss directly resulting from the discipline incident. Examples include the value of repairs necessitated by vandalism of a school facility, or the value of personnel resources used for repairs or consumed by the incident..</param>
        /// <param name="incidentDate">The month, day, and year on which the discipline incident occurred. (required).</param>
        /// <param name="incidentDescription">The description for an incident..</param>
        /// <param name="incidentLocationDescriptor">Identifies where the discipline incident occurred and whether or not it occurred on school..</param>
        /// <param name="incidentTime">An indication of the time of day the incident took place..</param>
        /// <param name="reportedToLawEnforcement">Indicator of whether the incident was reported to law enforcement..</param>
        /// <param name="reporterDescriptionDescriptor">Information on the type of individual who reported the discipline incident. When known and/or if useful, use a more specific option code (e.g., \&quot;Counselor\&quot; rather than \&quot;Professional Staff\&quot;)..</param>
        /// <param name="reporterName">Identifies the reporter of the discipline incident by name..</param>
        /// <param name="weapons">An unordered collection of disciplineIncidentWeapons. Identifies the type of weapon used during an incident. The Federal Gun-Free Schools Act requires states to report the number of students expelled for bringing firearms to school by type of firearm..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiDisciplineIncident(string id = default(string), string incidentIdentifier = default(string), EdFiSchoolReference schoolReference = default(EdFiSchoolReference), EdFiStaffReference staffReference = default(EdFiStaffReference), List<EdFiDisciplineIncidentBehavior> behaviors = default(List<EdFiDisciplineIncidentBehavior>), string caseNumber = default(string), List<EdFiDisciplineIncidentExternalParticipant> externalParticipants = default(List<EdFiDisciplineIncidentExternalParticipant>), double? incidentCost = default(double?), DateTime incidentDate = default(DateTime), string incidentDescription = default(string), string incidentLocationDescriptor = default(string), string incidentTime = default(string), bool? reportedToLawEnforcement = default(bool?), string reporterDescriptionDescriptor = default(string), string reporterName = default(string), List<EdFiDisciplineIncidentWeapon> weapons = default(List<EdFiDisciplineIncidentWeapon>), string etag = default(string))
        {
            // to ensure "incidentIdentifier" is required (not null)
            if (incidentIdentifier == null)
            {
                throw new ArgumentNullException("incidentIdentifier is a required property for EdFiDisciplineIncident and cannot be null");
            }
            this.IncidentIdentifier = incidentIdentifier;
            // to ensure "schoolReference" is required (not null)
            if (schoolReference == null)
            {
                throw new ArgumentNullException("schoolReference is a required property for EdFiDisciplineIncident and cannot be null");
            }
            this.SchoolReference = schoolReference;
            this.IncidentDate = incidentDate;
            this.Id = id;
            this.StaffReference = staffReference;
            this.Behaviors = behaviors;
            this.CaseNumber = caseNumber;
            this.ExternalParticipants = externalParticipants;
            this.IncidentCost = incidentCost;
            this.IncidentDescription = incidentDescription;
            this.IncidentLocationDescriptor = incidentLocationDescriptor;
            this.IncidentTime = incidentTime;
            this.ReportedToLawEnforcement = reportedToLawEnforcement;
            this.ReporterDescriptionDescriptor = reporterDescriptionDescriptor;
            this.ReporterName = reporterName;
            this.Weapons = weapons;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// A locally assigned unique identifier (within the school or school district) to identify each specific DisciplineIncident or occurrence. The same identifier should be used to document the entire discipline incident even if it included multiple offenses and multiple offenders.
        /// </summary>
        /// <value>A locally assigned unique identifier (within the school or school district) to identify each specific DisciplineIncident or occurrence. The same identifier should be used to document the entire discipline incident even if it included multiple offenses and multiple offenders.</value>
        [DataMember(Name = "incidentIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string IncidentIdentifier { get; set; }

        /// <summary>
        /// Gets or Sets SchoolReference
        /// </summary>
        [DataMember(Name = "schoolReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiSchoolReference SchoolReference { get; set; }

        /// <summary>
        /// Gets or Sets StaffReference
        /// </summary>
        [DataMember(Name = "staffReference", EmitDefaultValue = false)]
        public EdFiStaffReference StaffReference { get; set; }

        /// <summary>
        /// An unordered collection of disciplineIncidentBehaviors. Describes behavior by category and provides a detailed description.
        /// </summary>
        /// <value>An unordered collection of disciplineIncidentBehaviors. Describes behavior by category and provides a detailed description.</value>
        [DataMember(Name = "behaviors", EmitDefaultValue = false)]
        public List<EdFiDisciplineIncidentBehavior> Behaviors { get; set; }

        /// <summary>
        /// The case number assigned to the DisciplineIncident by law enforcement or other organization.
        /// </summary>
        /// <value>The case number assigned to the DisciplineIncident by law enforcement or other organization.</value>
        [DataMember(Name = "caseNumber", EmitDefaultValue = true)]
        public string CaseNumber { get; set; }

        /// <summary>
        /// An unordered collection of disciplineIncidentExternalParticipants. Information on an individual involved in the discipline incident.
        /// </summary>
        /// <value>An unordered collection of disciplineIncidentExternalParticipants. Information on an individual involved in the discipline incident.</value>
        [DataMember(Name = "externalParticipants", EmitDefaultValue = false)]
        public List<EdFiDisciplineIncidentExternalParticipant> ExternalParticipants { get; set; }

        /// <summary>
        /// The value of any quantifiable monetary loss directly resulting from the discipline incident. Examples include the value of repairs necessitated by vandalism of a school facility, or the value of personnel resources used for repairs or consumed by the incident.
        /// </summary>
        /// <value>The value of any quantifiable monetary loss directly resulting from the discipline incident. Examples include the value of repairs necessitated by vandalism of a school facility, or the value of personnel resources used for repairs or consumed by the incident.</value>
        [DataMember(Name = "incidentCost", EmitDefaultValue = true)]
        public double? IncidentCost { get; set; }

        /// <summary>
        /// The month, day, and year on which the discipline incident occurred.
        /// </summary>
        /// <value>The month, day, and year on which the discipline incident occurred.</value>
        [DataMember(Name = "incidentDate", IsRequired = true, EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime IncidentDate { get; set; }

        /// <summary>
        /// The description for an incident.
        /// </summary>
        /// <value>The description for an incident.</value>
        [DataMember(Name = "incidentDescription", EmitDefaultValue = true)]
        public string IncidentDescription { get; set; }

        /// <summary>
        /// Identifies where the discipline incident occurred and whether or not it occurred on school.
        /// </summary>
        /// <value>Identifies where the discipline incident occurred and whether or not it occurred on school.</value>
        [DataMember(Name = "incidentLocationDescriptor", EmitDefaultValue = true)]
        public string IncidentLocationDescriptor { get; set; }

        /// <summary>
        /// An indication of the time of day the incident took place.
        /// </summary>
        /// <value>An indication of the time of day the incident took place.</value>
        [DataMember(Name = "incidentTime", EmitDefaultValue = true)]
        public string IncidentTime { get; set; }

        /// <summary>
        /// Indicator of whether the incident was reported to law enforcement.
        /// </summary>
        /// <value>Indicator of whether the incident was reported to law enforcement.</value>
        [DataMember(Name = "reportedToLawEnforcement", EmitDefaultValue = true)]
        public bool? ReportedToLawEnforcement { get; set; }

        /// <summary>
        /// Information on the type of individual who reported the discipline incident. When known and/or if useful, use a more specific option code (e.g., \&quot;Counselor\&quot; rather than \&quot;Professional Staff\&quot;).
        /// </summary>
        /// <value>Information on the type of individual who reported the discipline incident. When known and/or if useful, use a more specific option code (e.g., \&quot;Counselor\&quot; rather than \&quot;Professional Staff\&quot;).</value>
        [DataMember(Name = "reporterDescriptionDescriptor", EmitDefaultValue = true)]
        public string ReporterDescriptionDescriptor { get; set; }

        /// <summary>
        /// Identifies the reporter of the discipline incident by name.
        /// </summary>
        /// <value>Identifies the reporter of the discipline incident by name.</value>
        [DataMember(Name = "reporterName", EmitDefaultValue = true)]
        public string ReporterName { get; set; }

        /// <summary>
        /// An unordered collection of disciplineIncidentWeapons. Identifies the type of weapon used during an incident. The Federal Gun-Free Schools Act requires states to report the number of students expelled for bringing firearms to school by type of firearm.
        /// </summary>
        /// <value>An unordered collection of disciplineIncidentWeapons. Identifies the type of weapon used during an incident. The Federal Gun-Free Schools Act requires states to report the number of students expelled for bringing firearms to school by type of firearm.</value>
        [DataMember(Name = "weapons", EmitDefaultValue = false)]
        public List<EdFiDisciplineIncidentWeapon> Weapons { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiDisciplineIncident {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  IncidentIdentifier: ").Append(IncidentIdentifier).Append("\n");
            sb.Append("  SchoolReference: ").Append(SchoolReference).Append("\n");
            sb.Append("  StaffReference: ").Append(StaffReference).Append("\n");
            sb.Append("  Behaviors: ").Append(Behaviors).Append("\n");
            sb.Append("  CaseNumber: ").Append(CaseNumber).Append("\n");
            sb.Append("  ExternalParticipants: ").Append(ExternalParticipants).Append("\n");
            sb.Append("  IncidentCost: ").Append(IncidentCost).Append("\n");
            sb.Append("  IncidentDate: ").Append(IncidentDate).Append("\n");
            sb.Append("  IncidentDescription: ").Append(IncidentDescription).Append("\n");
            sb.Append("  IncidentLocationDescriptor: ").Append(IncidentLocationDescriptor).Append("\n");
            sb.Append("  IncidentTime: ").Append(IncidentTime).Append("\n");
            sb.Append("  ReportedToLawEnforcement: ").Append(ReportedToLawEnforcement).Append("\n");
            sb.Append("  ReporterDescriptionDescriptor: ").Append(ReporterDescriptionDescriptor).Append("\n");
            sb.Append("  ReporterName: ").Append(ReporterName).Append("\n");
            sb.Append("  Weapons: ").Append(Weapons).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiDisciplineIncident);
        }

        /// <summary>
        /// Returns true if EdFiDisciplineIncident instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiDisciplineIncident to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiDisciplineIncident input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.IncidentIdentifier == input.IncidentIdentifier ||
                    (this.IncidentIdentifier != null &&
                    this.IncidentIdentifier.Equals(input.IncidentIdentifier))
                ) && 
                (
                    this.SchoolReference == input.SchoolReference ||
                    (this.SchoolReference != null &&
                    this.SchoolReference.Equals(input.SchoolReference))
                ) && 
                (
                    this.StaffReference == input.StaffReference ||
                    (this.StaffReference != null &&
                    this.StaffReference.Equals(input.StaffReference))
                ) && 
                (
                    this.Behaviors == input.Behaviors ||
                    this.Behaviors != null &&
                    input.Behaviors != null &&
                    this.Behaviors.SequenceEqual(input.Behaviors)
                ) && 
                (
                    this.CaseNumber == input.CaseNumber ||
                    (this.CaseNumber != null &&
                    this.CaseNumber.Equals(input.CaseNumber))
                ) && 
                (
                    this.ExternalParticipants == input.ExternalParticipants ||
                    this.ExternalParticipants != null &&
                    input.ExternalParticipants != null &&
                    this.ExternalParticipants.SequenceEqual(input.ExternalParticipants)
                ) && 
                (
                    this.IncidentCost == input.IncidentCost ||
                    (this.IncidentCost != null &&
                    this.IncidentCost.Equals(input.IncidentCost))
                ) && 
                (
                    this.IncidentDate == input.IncidentDate ||
                    (this.IncidentDate != null &&
                    this.IncidentDate.Equals(input.IncidentDate))
                ) && 
                (
                    this.IncidentDescription == input.IncidentDescription ||
                    (this.IncidentDescription != null &&
                    this.IncidentDescription.Equals(input.IncidentDescription))
                ) && 
                (
                    this.IncidentLocationDescriptor == input.IncidentLocationDescriptor ||
                    (this.IncidentLocationDescriptor != null &&
                    this.IncidentLocationDescriptor.Equals(input.IncidentLocationDescriptor))
                ) && 
                (
                    this.IncidentTime == input.IncidentTime ||
                    (this.IncidentTime != null &&
                    this.IncidentTime.Equals(input.IncidentTime))
                ) && 
                (
                    this.ReportedToLawEnforcement == input.ReportedToLawEnforcement ||
                    (this.ReportedToLawEnforcement != null &&
                    this.ReportedToLawEnforcement.Equals(input.ReportedToLawEnforcement))
                ) && 
                (
                    this.ReporterDescriptionDescriptor == input.ReporterDescriptionDescriptor ||
                    (this.ReporterDescriptionDescriptor != null &&
                    this.ReporterDescriptionDescriptor.Equals(input.ReporterDescriptionDescriptor))
                ) && 
                (
                    this.ReporterName == input.ReporterName ||
                    (this.ReporterName != null &&
                    this.ReporterName.Equals(input.ReporterName))
                ) && 
                (
                    this.Weapons == input.Weapons ||
                    this.Weapons != null &&
                    input.Weapons != null &&
                    this.Weapons.SequenceEqual(input.Weapons)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.IncidentIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentIdentifier.GetHashCode();
                }
                if (this.SchoolReference != null)
                {
                    hashCode = (hashCode * 59) + this.SchoolReference.GetHashCode();
                }
                if (this.StaffReference != null)
                {
                    hashCode = (hashCode * 59) + this.StaffReference.GetHashCode();
                }
                if (this.Behaviors != null)
                {
                    hashCode = (hashCode * 59) + this.Behaviors.GetHashCode();
                }
                if (this.CaseNumber != null)
                {
                    hashCode = (hashCode * 59) + this.CaseNumber.GetHashCode();
                }
                if (this.ExternalParticipants != null)
                {
                    hashCode = (hashCode * 59) + this.ExternalParticipants.GetHashCode();
                }
                if (this.IncidentCost != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentCost.GetHashCode();
                }
                if (this.IncidentDate != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentDate.GetHashCode();
                }
                if (this.IncidentDescription != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentDescription.GetHashCode();
                }
                if (this.IncidentLocationDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentLocationDescriptor.GetHashCode();
                }
                if (this.IncidentTime != null)
                {
                    hashCode = (hashCode * 59) + this.IncidentTime.GetHashCode();
                }
                if (this.ReportedToLawEnforcement != null)
                {
                    hashCode = (hashCode * 59) + this.ReportedToLawEnforcement.GetHashCode();
                }
                if (this.ReporterDescriptionDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ReporterDescriptionDescriptor.GetHashCode();
                }
                if (this.ReporterName != null)
                {
                    hashCode = (hashCode * 59) + this.ReporterName.GetHashCode();
                }
                if (this.Weapons != null)
                {
                    hashCode = (hashCode * 59) + this.Weapons.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // IncidentIdentifier (string) maxLength
            if (this.IncidentIdentifier != null && this.IncidentIdentifier.Length > 20)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for IncidentIdentifier, length must be less than 20.", new [] { "IncidentIdentifier" });
            }

            // CaseNumber (string) maxLength
            if (this.CaseNumber != null && this.CaseNumber.Length > 20)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CaseNumber, length must be less than 20.", new [] { "CaseNumber" });
            }

            // IncidentDescription (string) maxLength
            if (this.IncidentDescription != null && this.IncidentDescription.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for IncidentDescription, length must be less than 1024.", new [] { "IncidentDescription" });
            }

            // IncidentLocationDescriptor (string) maxLength
            if (this.IncidentLocationDescriptor != null && this.IncidentLocationDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for IncidentLocationDescriptor, length must be less than 306.", new [] { "IncidentLocationDescriptor" });
            }

            // ReporterDescriptionDescriptor (string) maxLength
            if (this.ReporterDescriptionDescriptor != null && this.ReporterDescriptionDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ReporterDescriptionDescriptor, length must be less than 306.", new [] { "ReporterDescriptionDescriptor" });
            }

            // ReporterName (string) maxLength
            if (this.ReporterName != null && this.ReporterName.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ReporterName, length must be less than 75.", new [] { "ReporterName" });
            }

            yield break;
        }
    }

}
