/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiGradebookEntry
    /// </summary>
    [DataContract(Name = "edFi_gradebookEntry")]
    public partial class EdFiGradebookEntry : IEquatable<EdFiGradebookEntry>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiGradebookEntry" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiGradebookEntry() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiGradebookEntry" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="gradebookEntryIdentifier">A unique number or alphanumeric code assigned to a gradebook entry by the source system. (required).</param>
        /// <param name="_namespace">Namespace URI for the source of the gradebook entry. (required).</param>
        /// <param name="gradingPeriodReference">gradingPeriodReference.</param>
        /// <param name="sectionReference">sectionReference.</param>
        /// <param name="dateAssigned">The date the assignment, homework, or assessment was assigned or executed. (required).</param>
        /// <param name="description">A description of the assignment, homework, or classroom assessment..</param>
        /// <param name="dueDate">The date the assignment, homework, or assessment is due..</param>
        /// <param name="dueTime">The time the assignment, homework, or assessment is due..</param>
        /// <param name="gradebookEntryTypeDescriptor">The type of the gradebook entry..</param>
        /// <param name="learningStandards">An unordered collection of gradebookEntryLearningStandards. LearningStandard(s) associated with the gradebook entry..</param>
        /// <param name="maxPoints">The maximum number of points  that can be earned for the submission..</param>
        /// <param name="sourceSectionIdentifier">The local identifier assigned to a section. (required).</param>
        /// <param name="title">The name or title of the activity to be recorded in the gradebook entry. (required).</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiGradebookEntry(string id = default(string), string gradebookEntryIdentifier = default(string), string _namespace = default(string), EdFiGradingPeriodReference gradingPeriodReference = default(EdFiGradingPeriodReference), EdFiSectionReference sectionReference = default(EdFiSectionReference), DateTime dateAssigned = default(DateTime), string description = default(string), DateTime? dueDate = default(DateTime?), string dueTime = default(string), string gradebookEntryTypeDescriptor = default(string), List<EdFiGradebookEntryLearningStandard> learningStandards = default(List<EdFiGradebookEntryLearningStandard>), double? maxPoints = default(double?), string sourceSectionIdentifier = default(string), string title = default(string), string etag = default(string))
        {
            // to ensure "gradebookEntryIdentifier" is required (not null)
            if (gradebookEntryIdentifier == null)
            {
                throw new ArgumentNullException("gradebookEntryIdentifier is a required property for EdFiGradebookEntry and cannot be null");
            }
            this.GradebookEntryIdentifier = gradebookEntryIdentifier;
            // to ensure "_namespace" is required (not null)
            if (_namespace == null)
            {
                throw new ArgumentNullException("_namespace is a required property for EdFiGradebookEntry and cannot be null");
            }
            this.Namespace = _namespace;
            this.DateAssigned = dateAssigned;
            // to ensure "sourceSectionIdentifier" is required (not null)
            if (sourceSectionIdentifier == null)
            {
                throw new ArgumentNullException("sourceSectionIdentifier is a required property for EdFiGradebookEntry and cannot be null");
            }
            this.SourceSectionIdentifier = sourceSectionIdentifier;
            // to ensure "title" is required (not null)
            if (title == null)
            {
                throw new ArgumentNullException("title is a required property for EdFiGradebookEntry and cannot be null");
            }
            this.Title = title;
            this.Id = id;
            this.GradingPeriodReference = gradingPeriodReference;
            this.SectionReference = sectionReference;
            this.Description = description;
            this.DueDate = dueDate;
            this.DueTime = dueTime;
            this.GradebookEntryTypeDescriptor = gradebookEntryTypeDescriptor;
            this.LearningStandards = learningStandards;
            this.MaxPoints = maxPoints;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to a gradebook entry by the source system.
        /// </summary>
        /// <value>A unique number or alphanumeric code assigned to a gradebook entry by the source system.</value>
        [DataMember(Name = "gradebookEntryIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string GradebookEntryIdentifier { get; set; }

        /// <summary>
        /// Namespace URI for the source of the gradebook entry.
        /// </summary>
        /// <value>Namespace URI for the source of the gradebook entry.</value>
        [DataMember(Name = "namespace", IsRequired = true, EmitDefaultValue = true)]
        public string Namespace { get; set; }

        /// <summary>
        /// Gets or Sets GradingPeriodReference
        /// </summary>
        [DataMember(Name = "gradingPeriodReference", EmitDefaultValue = false)]
        public EdFiGradingPeriodReference GradingPeriodReference { get; set; }

        /// <summary>
        /// Gets or Sets SectionReference
        /// </summary>
        [DataMember(Name = "sectionReference", EmitDefaultValue = false)]
        public EdFiSectionReference SectionReference { get; set; }

        /// <summary>
        /// The date the assignment, homework, or assessment was assigned or executed.
        /// </summary>
        /// <value>The date the assignment, homework, or assessment was assigned or executed.</value>
        [DataMember(Name = "dateAssigned", IsRequired = true, EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime DateAssigned { get; set; }

        /// <summary>
        /// A description of the assignment, homework, or classroom assessment.
        /// </summary>
        /// <value>A description of the assignment, homework, or classroom assessment.</value>
        [DataMember(Name = "description", EmitDefaultValue = true)]
        public string Description { get; set; }

        /// <summary>
        /// The date the assignment, homework, or assessment is due.
        /// </summary>
        /// <value>The date the assignment, homework, or assessment is due.</value>
        [DataMember(Name = "dueDate", EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime? DueDate { get; set; }

        /// <summary>
        /// The time the assignment, homework, or assessment is due.
        /// </summary>
        /// <value>The time the assignment, homework, or assessment is due.</value>
        [DataMember(Name = "dueTime", EmitDefaultValue = true)]
        public string DueTime { get; set; }

        /// <summary>
        /// The type of the gradebook entry.
        /// </summary>
        /// <value>The type of the gradebook entry.</value>
        [DataMember(Name = "gradebookEntryTypeDescriptor", EmitDefaultValue = true)]
        public string GradebookEntryTypeDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of gradebookEntryLearningStandards. LearningStandard(s) associated with the gradebook entry.
        /// </summary>
        /// <value>An unordered collection of gradebookEntryLearningStandards. LearningStandard(s) associated with the gradebook entry.</value>
        [DataMember(Name = "learningStandards", EmitDefaultValue = false)]
        public List<EdFiGradebookEntryLearningStandard> LearningStandards { get; set; }

        /// <summary>
        /// The maximum number of points  that can be earned for the submission.
        /// </summary>
        /// <value>The maximum number of points  that can be earned for the submission.</value>
        [DataMember(Name = "maxPoints", EmitDefaultValue = true)]
        public double? MaxPoints { get; set; }

        /// <summary>
        /// The local identifier assigned to a section.
        /// </summary>
        /// <value>The local identifier assigned to a section.</value>
        [DataMember(Name = "sourceSectionIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string SourceSectionIdentifier { get; set; }

        /// <summary>
        /// The name or title of the activity to be recorded in the gradebook entry.
        /// </summary>
        /// <value>The name or title of the activity to be recorded in the gradebook entry.</value>
        [DataMember(Name = "title", IsRequired = true, EmitDefaultValue = true)]
        public string Title { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiGradebookEntry {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  GradebookEntryIdentifier: ").Append(GradebookEntryIdentifier).Append("\n");
            sb.Append("  Namespace: ").Append(Namespace).Append("\n");
            sb.Append("  GradingPeriodReference: ").Append(GradingPeriodReference).Append("\n");
            sb.Append("  SectionReference: ").Append(SectionReference).Append("\n");
            sb.Append("  DateAssigned: ").Append(DateAssigned).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  DueDate: ").Append(DueDate).Append("\n");
            sb.Append("  DueTime: ").Append(DueTime).Append("\n");
            sb.Append("  GradebookEntryTypeDescriptor: ").Append(GradebookEntryTypeDescriptor).Append("\n");
            sb.Append("  LearningStandards: ").Append(LearningStandards).Append("\n");
            sb.Append("  MaxPoints: ").Append(MaxPoints).Append("\n");
            sb.Append("  SourceSectionIdentifier: ").Append(SourceSectionIdentifier).Append("\n");
            sb.Append("  Title: ").Append(Title).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiGradebookEntry);
        }

        /// <summary>
        /// Returns true if EdFiGradebookEntry instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiGradebookEntry to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiGradebookEntry input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.GradebookEntryIdentifier == input.GradebookEntryIdentifier ||
                    (this.GradebookEntryIdentifier != null &&
                    this.GradebookEntryIdentifier.Equals(input.GradebookEntryIdentifier))
                ) && 
                (
                    this.Namespace == input.Namespace ||
                    (this.Namespace != null &&
                    this.Namespace.Equals(input.Namespace))
                ) && 
                (
                    this.GradingPeriodReference == input.GradingPeriodReference ||
                    (this.GradingPeriodReference != null &&
                    this.GradingPeriodReference.Equals(input.GradingPeriodReference))
                ) && 
                (
                    this.SectionReference == input.SectionReference ||
                    (this.SectionReference != null &&
                    this.SectionReference.Equals(input.SectionReference))
                ) && 
                (
                    this.DateAssigned == input.DateAssigned ||
                    (this.DateAssigned != null &&
                    this.DateAssigned.Equals(input.DateAssigned))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.DueDate == input.DueDate ||
                    (this.DueDate != null &&
                    this.DueDate.Equals(input.DueDate))
                ) && 
                (
                    this.DueTime == input.DueTime ||
                    (this.DueTime != null &&
                    this.DueTime.Equals(input.DueTime))
                ) && 
                (
                    this.GradebookEntryTypeDescriptor == input.GradebookEntryTypeDescriptor ||
                    (this.GradebookEntryTypeDescriptor != null &&
                    this.GradebookEntryTypeDescriptor.Equals(input.GradebookEntryTypeDescriptor))
                ) && 
                (
                    this.LearningStandards == input.LearningStandards ||
                    this.LearningStandards != null &&
                    input.LearningStandards != null &&
                    this.LearningStandards.SequenceEqual(input.LearningStandards)
                ) && 
                (
                    this.MaxPoints == input.MaxPoints ||
                    (this.MaxPoints != null &&
                    this.MaxPoints.Equals(input.MaxPoints))
                ) && 
                (
                    this.SourceSectionIdentifier == input.SourceSectionIdentifier ||
                    (this.SourceSectionIdentifier != null &&
                    this.SourceSectionIdentifier.Equals(input.SourceSectionIdentifier))
                ) && 
                (
                    this.Title == input.Title ||
                    (this.Title != null &&
                    this.Title.Equals(input.Title))
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.GradebookEntryIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.GradebookEntryIdentifier.GetHashCode();
                }
                if (this.Namespace != null)
                {
                    hashCode = (hashCode * 59) + this.Namespace.GetHashCode();
                }
                if (this.GradingPeriodReference != null)
                {
                    hashCode = (hashCode * 59) + this.GradingPeriodReference.GetHashCode();
                }
                if (this.SectionReference != null)
                {
                    hashCode = (hashCode * 59) + this.SectionReference.GetHashCode();
                }
                if (this.DateAssigned != null)
                {
                    hashCode = (hashCode * 59) + this.DateAssigned.GetHashCode();
                }
                if (this.Description != null)
                {
                    hashCode = (hashCode * 59) + this.Description.GetHashCode();
                }
                if (this.DueDate != null)
                {
                    hashCode = (hashCode * 59) + this.DueDate.GetHashCode();
                }
                if (this.DueTime != null)
                {
                    hashCode = (hashCode * 59) + this.DueTime.GetHashCode();
                }
                if (this.GradebookEntryTypeDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.GradebookEntryTypeDescriptor.GetHashCode();
                }
                if (this.LearningStandards != null)
                {
                    hashCode = (hashCode * 59) + this.LearningStandards.GetHashCode();
                }
                if (this.MaxPoints != null)
                {
                    hashCode = (hashCode * 59) + this.MaxPoints.GetHashCode();
                }
                if (this.SourceSectionIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.SourceSectionIdentifier.GetHashCode();
                }
                if (this.Title != null)
                {
                    hashCode = (hashCode * 59) + this.Title.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // GradebookEntryIdentifier (string) maxLength
            if (this.GradebookEntryIdentifier != null && this.GradebookEntryIdentifier.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for GradebookEntryIdentifier, length must be less than 60.", new [] { "GradebookEntryIdentifier" });
            }

            // Namespace (string) maxLength
            if (this.Namespace != null && this.Namespace.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Namespace, length must be less than 255.", new [] { "Namespace" });
            }

            // Description (string) maxLength
            if (this.Description != null && this.Description.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Description, length must be less than 1024.", new [] { "Description" });
            }

            // GradebookEntryTypeDescriptor (string) maxLength
            if (this.GradebookEntryTypeDescriptor != null && this.GradebookEntryTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for GradebookEntryTypeDescriptor, length must be less than 306.", new [] { "GradebookEntryTypeDescriptor" });
            }

            // SourceSectionIdentifier (string) maxLength
            if (this.SourceSectionIdentifier != null && this.SourceSectionIdentifier.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SourceSectionIdentifier, length must be less than 255.", new [] { "SourceSectionIdentifier" });
            }

            // Title (string) maxLength
            if (this.Title != null && this.Title.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Title, length must be less than 100.", new [] { "Title" });
            }

            yield break;
        }
    }

}
