/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiLocalAccount
    /// </summary>
    [DataContract(Name = "edFi_localAccount")]
    public partial class EdFiLocalAccount : IEquatable<EdFiLocalAccount>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiLocalAccount" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiLocalAccount() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiLocalAccount" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="accountIdentifier">Code value for the valid combination of account dimensions by LEA under which financials are reported. (required).</param>
        /// <param name="fiscalYear">The fiscal year for the account. (required).</param>
        /// <param name="chartOfAccountReference">chartOfAccountReference (required).</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="accountName">A descriptive name for the account..</param>
        /// <param name="reportingTags">An unordered collection of localAccountReportingTags. Optional tag for accountability reporting..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiLocalAccount(string id = default(string), string accountIdentifier = default(string), int fiscalYear = default(int), EdFiChartOfAccountReference chartOfAccountReference = default(EdFiChartOfAccountReference), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), string accountName = default(string), List<EdFiLocalAccountReportingTag> reportingTags = default(List<EdFiLocalAccountReportingTag>), string etag = default(string))
        {
            // to ensure "accountIdentifier" is required (not null)
            if (accountIdentifier == null)
            {
                throw new ArgumentNullException("accountIdentifier is a required property for EdFiLocalAccount and cannot be null");
            }
            this.AccountIdentifier = accountIdentifier;
            this.FiscalYear = fiscalYear;
            // to ensure "chartOfAccountReference" is required (not null)
            if (chartOfAccountReference == null)
            {
                throw new ArgumentNullException("chartOfAccountReference is a required property for EdFiLocalAccount and cannot be null");
            }
            this.ChartOfAccountReference = chartOfAccountReference;
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new ArgumentNullException("educationOrganizationReference is a required property for EdFiLocalAccount and cannot be null");
            }
            this.EducationOrganizationReference = educationOrganizationReference;
            this.Id = id;
            this.AccountName = accountName;
            this.ReportingTags = reportingTags;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// Code value for the valid combination of account dimensions by LEA under which financials are reported.
        /// </summary>
        /// <value>Code value for the valid combination of account dimensions by LEA under which financials are reported.</value>
        [DataMember(Name = "accountIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string AccountIdentifier { get; set; }

        /// <summary>
        /// The fiscal year for the account.
        /// </summary>
        /// <value>The fiscal year for the account.</value>
        [DataMember(Name = "fiscalYear", IsRequired = true, EmitDefaultValue = true)]
        public int FiscalYear { get; set; }

        /// <summary>
        /// Gets or Sets ChartOfAccountReference
        /// </summary>
        [DataMember(Name = "chartOfAccountReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiChartOfAccountReference ChartOfAccountReference { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name = "educationOrganizationReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// A descriptive name for the account.
        /// </summary>
        /// <value>A descriptive name for the account.</value>
        [DataMember(Name = "accountName", EmitDefaultValue = true)]
        public string AccountName { get; set; }

        /// <summary>
        /// An unordered collection of localAccountReportingTags. Optional tag for accountability reporting.
        /// </summary>
        /// <value>An unordered collection of localAccountReportingTags. Optional tag for accountability reporting.</value>
        [DataMember(Name = "reportingTags", EmitDefaultValue = false)]
        public List<EdFiLocalAccountReportingTag> ReportingTags { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiLocalAccount {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  AccountIdentifier: ").Append(AccountIdentifier).Append("\n");
            sb.Append("  FiscalYear: ").Append(FiscalYear).Append("\n");
            sb.Append("  ChartOfAccountReference: ").Append(ChartOfAccountReference).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  AccountName: ").Append(AccountName).Append("\n");
            sb.Append("  ReportingTags: ").Append(ReportingTags).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiLocalAccount);
        }

        /// <summary>
        /// Returns true if EdFiLocalAccount instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiLocalAccount to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiLocalAccount input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.AccountIdentifier == input.AccountIdentifier ||
                    (this.AccountIdentifier != null &&
                    this.AccountIdentifier.Equals(input.AccountIdentifier))
                ) && 
                (
                    this.FiscalYear == input.FiscalYear ||
                    this.FiscalYear.Equals(input.FiscalYear)
                ) && 
                (
                    this.ChartOfAccountReference == input.ChartOfAccountReference ||
                    (this.ChartOfAccountReference != null &&
                    this.ChartOfAccountReference.Equals(input.ChartOfAccountReference))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.AccountName == input.AccountName ||
                    (this.AccountName != null &&
                    this.AccountName.Equals(input.AccountName))
                ) && 
                (
                    this.ReportingTags == input.ReportingTags ||
                    this.ReportingTags != null &&
                    input.ReportingTags != null &&
                    this.ReportingTags.SequenceEqual(input.ReportingTags)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.AccountIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.AccountIdentifier.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.FiscalYear.GetHashCode();
                if (this.ChartOfAccountReference != null)
                {
                    hashCode = (hashCode * 59) + this.ChartOfAccountReference.GetHashCode();
                }
                if (this.EducationOrganizationReference != null)
                {
                    hashCode = (hashCode * 59) + this.EducationOrganizationReference.GetHashCode();
                }
                if (this.AccountName != null)
                {
                    hashCode = (hashCode * 59) + this.AccountName.GetHashCode();
                }
                if (this.ReportingTags != null)
                {
                    hashCode = (hashCode * 59) + this.ReportingTags.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // AccountIdentifier (string) maxLength
            if (this.AccountIdentifier != null && this.AccountIdentifier.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountIdentifier, length must be less than 50.", new [] { "AccountIdentifier" });
            }

            // AccountName (string) maxLength
            if (this.AccountName != null && this.AccountName.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AccountName, length must be less than 100.", new [] { "AccountName" });
            }

            yield break;
        }
    }

}
