/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiProgram
    /// </summary>
    [DataContract(Name = "edFi_program")]
    public partial class EdFiProgram : IEquatable<EdFiProgram>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiProgram" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiProgram() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiProgram" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="programName">The formal name of the program of instruction, training, services, or benefits available through federal, state, or local agencies. (required).</param>
        /// <param name="programTypeDescriptor">The type of program. (required).</param>
        /// <param name="educationOrganizationReference">educationOrganizationReference (required).</param>
        /// <param name="characteristics">An unordered collection of programCharacteristics. Reflects important characteristics of the program, such as categories or particular indications..</param>
        /// <param name="learningObjectives">An unordered collection of programLearningObjectives. References the learning objective(s) with which the program is associated..</param>
        /// <param name="learningStandards">An unordered collection of programLearningStandards. Learning standard followed by this program..</param>
        /// <param name="programId">A unique number or alphanumeric code assigned to a program by a school, school system, a state, or other agency or entity..</param>
        /// <param name="services">An unordered collection of programServices. Defines the services this program provides to students..</param>
        /// <param name="sponsors">An unordered collection of programSponsors. Ultimate and intermediate providers of funds for a particular educational or service program or activity, or for an individual&#39;s participation in the program or activity (e.g., Federal, State, ESC, District, School, Private Organization)..</param>
        /// <param name="etag">A unique system-generated value that identifies the version of the resource..</param>
        public EdFiProgram(string id = default(string), string programName = default(string), string programTypeDescriptor = default(string), EdFiEducationOrganizationReference educationOrganizationReference = default(EdFiEducationOrganizationReference), List<EdFiProgramCharacteristic> characteristics = default(List<EdFiProgramCharacteristic>), List<EdFiProgramLearningObjective> learningObjectives = default(List<EdFiProgramLearningObjective>), List<EdFiProgramLearningStandard> learningStandards = default(List<EdFiProgramLearningStandard>), string programId = default(string), List<EdFiProgramService> services = default(List<EdFiProgramService>), List<EdFiProgramSponsor> sponsors = default(List<EdFiProgramSponsor>), string etag = default(string))
        {
            // to ensure "programName" is required (not null)
            if (programName == null)
            {
                throw new ArgumentNullException("programName is a required property for EdFiProgram and cannot be null");
            }
            this.ProgramName = programName;
            // to ensure "programTypeDescriptor" is required (not null)
            if (programTypeDescriptor == null)
            {
                throw new ArgumentNullException("programTypeDescriptor is a required property for EdFiProgram and cannot be null");
            }
            this.ProgramTypeDescriptor = programTypeDescriptor;
            // to ensure "educationOrganizationReference" is required (not null)
            if (educationOrganizationReference == null)
            {
                throw new ArgumentNullException("educationOrganizationReference is a required property for EdFiProgram and cannot be null");
            }
            this.EducationOrganizationReference = educationOrganizationReference;
            this.Id = id;
            this.Characteristics = characteristics;
            this.LearningObjectives = learningObjectives;
            this.LearningStandards = learningStandards;
            this.ProgramId = programId;
            this.Services = services;
            this.Sponsors = sponsors;
            this.Etag = etag;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// The formal name of the program of instruction, training, services, or benefits available through federal, state, or local agencies.
        /// </summary>
        /// <value>The formal name of the program of instruction, training, services, or benefits available through federal, state, or local agencies.</value>
        [DataMember(Name = "programName", IsRequired = true, EmitDefaultValue = true)]
        public string ProgramName { get; set; }

        /// <summary>
        /// The type of program.
        /// </summary>
        /// <value>The type of program.</value>
        [DataMember(Name = "programTypeDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ProgramTypeDescriptor { get; set; }

        /// <summary>
        /// Gets or Sets EducationOrganizationReference
        /// </summary>
        [DataMember(Name = "educationOrganizationReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiEducationOrganizationReference EducationOrganizationReference { get; set; }

        /// <summary>
        /// An unordered collection of programCharacteristics. Reflects important characteristics of the program, such as categories or particular indications.
        /// </summary>
        /// <value>An unordered collection of programCharacteristics. Reflects important characteristics of the program, such as categories or particular indications.</value>
        [DataMember(Name = "characteristics", EmitDefaultValue = false)]
        public List<EdFiProgramCharacteristic> Characteristics { get; set; }

        /// <summary>
        /// An unordered collection of programLearningObjectives. References the learning objective(s) with which the program is associated.
        /// </summary>
        /// <value>An unordered collection of programLearningObjectives. References the learning objective(s) with which the program is associated.</value>
        [DataMember(Name = "learningObjectives", EmitDefaultValue = false)]
        public List<EdFiProgramLearningObjective> LearningObjectives { get; set; }

        /// <summary>
        /// An unordered collection of programLearningStandards. Learning standard followed by this program.
        /// </summary>
        /// <value>An unordered collection of programLearningStandards. Learning standard followed by this program.</value>
        [DataMember(Name = "learningStandards", EmitDefaultValue = false)]
        public List<EdFiProgramLearningStandard> LearningStandards { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to a program by a school, school system, a state, or other agency or entity.
        /// </summary>
        /// <value>A unique number or alphanumeric code assigned to a program by a school, school system, a state, or other agency or entity.</value>
        [DataMember(Name = "programId", EmitDefaultValue = true)]
        public string ProgramId { get; set; }

        /// <summary>
        /// An unordered collection of programServices. Defines the services this program provides to students.
        /// </summary>
        /// <value>An unordered collection of programServices. Defines the services this program provides to students.</value>
        [DataMember(Name = "services", EmitDefaultValue = false)]
        public List<EdFiProgramService> Services { get; set; }

        /// <summary>
        /// An unordered collection of programSponsors. Ultimate and intermediate providers of funds for a particular educational or service program or activity, or for an individual&#39;s participation in the program or activity (e.g., Federal, State, ESC, District, School, Private Organization).
        /// </summary>
        /// <value>An unordered collection of programSponsors. Ultimate and intermediate providers of funds for a particular educational or service program or activity, or for an individual&#39;s participation in the program or activity (e.g., Federal, State, ESC, District, School, Private Organization).</value>
        [DataMember(Name = "sponsors", EmitDefaultValue = false)]
        public List<EdFiProgramSponsor> Sponsors { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        /// <value>A unique system-generated value that identifies the version of the resource.</value>
        [DataMember(Name = "_etag", EmitDefaultValue = false)]
        public string Etag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiProgram {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  ProgramName: ").Append(ProgramName).Append("\n");
            sb.Append("  ProgramTypeDescriptor: ").Append(ProgramTypeDescriptor).Append("\n");
            sb.Append("  EducationOrganizationReference: ").Append(EducationOrganizationReference).Append("\n");
            sb.Append("  Characteristics: ").Append(Characteristics).Append("\n");
            sb.Append("  LearningObjectives: ").Append(LearningObjectives).Append("\n");
            sb.Append("  LearningStandards: ").Append(LearningStandards).Append("\n");
            sb.Append("  ProgramId: ").Append(ProgramId).Append("\n");
            sb.Append("  Services: ").Append(Services).Append("\n");
            sb.Append("  Sponsors: ").Append(Sponsors).Append("\n");
            sb.Append("  Etag: ").Append(Etag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiProgram);
        }

        /// <summary>
        /// Returns true if EdFiProgram instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiProgram to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiProgram input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.ProgramName == input.ProgramName ||
                    (this.ProgramName != null &&
                    this.ProgramName.Equals(input.ProgramName))
                ) && 
                (
                    this.ProgramTypeDescriptor == input.ProgramTypeDescriptor ||
                    (this.ProgramTypeDescriptor != null &&
                    this.ProgramTypeDescriptor.Equals(input.ProgramTypeDescriptor))
                ) && 
                (
                    this.EducationOrganizationReference == input.EducationOrganizationReference ||
                    (this.EducationOrganizationReference != null &&
                    this.EducationOrganizationReference.Equals(input.EducationOrganizationReference))
                ) && 
                (
                    this.Characteristics == input.Characteristics ||
                    this.Characteristics != null &&
                    input.Characteristics != null &&
                    this.Characteristics.SequenceEqual(input.Characteristics)
                ) && 
                (
                    this.LearningObjectives == input.LearningObjectives ||
                    this.LearningObjectives != null &&
                    input.LearningObjectives != null &&
                    this.LearningObjectives.SequenceEqual(input.LearningObjectives)
                ) && 
                (
                    this.LearningStandards == input.LearningStandards ||
                    this.LearningStandards != null &&
                    input.LearningStandards != null &&
                    this.LearningStandards.SequenceEqual(input.LearningStandards)
                ) && 
                (
                    this.ProgramId == input.ProgramId ||
                    (this.ProgramId != null &&
                    this.ProgramId.Equals(input.ProgramId))
                ) && 
                (
                    this.Services == input.Services ||
                    this.Services != null &&
                    input.Services != null &&
                    this.Services.SequenceEqual(input.Services)
                ) && 
                (
                    this.Sponsors == input.Sponsors ||
                    this.Sponsors != null &&
                    input.Sponsors != null &&
                    this.Sponsors.SequenceEqual(input.Sponsors)
                ) && 
                (
                    this.Etag == input.Etag ||
                    (this.Etag != null &&
                    this.Etag.Equals(input.Etag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.ProgramName != null)
                {
                    hashCode = (hashCode * 59) + this.ProgramName.GetHashCode();
                }
                if (this.ProgramTypeDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ProgramTypeDescriptor.GetHashCode();
                }
                if (this.EducationOrganizationReference != null)
                {
                    hashCode = (hashCode * 59) + this.EducationOrganizationReference.GetHashCode();
                }
                if (this.Characteristics != null)
                {
                    hashCode = (hashCode * 59) + this.Characteristics.GetHashCode();
                }
                if (this.LearningObjectives != null)
                {
                    hashCode = (hashCode * 59) + this.LearningObjectives.GetHashCode();
                }
                if (this.LearningStandards != null)
                {
                    hashCode = (hashCode * 59) + this.LearningStandards.GetHashCode();
                }
                if (this.ProgramId != null)
                {
                    hashCode = (hashCode * 59) + this.ProgramId.GetHashCode();
                }
                if (this.Services != null)
                {
                    hashCode = (hashCode * 59) + this.Services.GetHashCode();
                }
                if (this.Sponsors != null)
                {
                    hashCode = (hashCode * 59) + this.Sponsors.GetHashCode();
                }
                if (this.Etag != null)
                {
                    hashCode = (hashCode * 59) + this.Etag.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // ProgramName (string) maxLength
            if (this.ProgramName != null && this.ProgramName.Length > 60)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProgramName, length must be less than 60.", new [] { "ProgramName" });
            }

            // ProgramTypeDescriptor (string) maxLength
            if (this.ProgramTypeDescriptor != null && this.ProgramTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProgramTypeDescriptor, length must be less than 306.", new [] { "ProgramTypeDescriptor" });
            }

            // ProgramId (string) maxLength
            if (this.ProgramId != null && this.ProgramId.Length > 20)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ProgramId, length must be less than 20.", new [] { "ProgramId" });
            }

            yield break;
        }
    }

}
