/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiProgramService
    /// </summary>
    [DataContract(Name = "edFi_programService")]
    public partial class EdFiProgramService : IEquatable<EdFiProgramService>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiProgramService" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiProgramService() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiProgramService" /> class.
        /// </summary>
        /// <param name="serviceDescriptor">Defines the services this program provides to students. (required).</param>
        public EdFiProgramService(string serviceDescriptor = default(string))
        {
            // to ensure "serviceDescriptor" is required (not null)
            if (serviceDescriptor == null)
            {
                throw new ArgumentNullException("serviceDescriptor is a required property for EdFiProgramService and cannot be null");
            }
            this.ServiceDescriptor = serviceDescriptor;
        }

        /// <summary>
        /// Defines the services this program provides to students.
        /// </summary>
        /// <value>Defines the services this program provides to students.</value>
        [DataMember(Name = "serviceDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string ServiceDescriptor { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiProgramService {\n");
            sb.Append("  ServiceDescriptor: ").Append(ServiceDescriptor).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiProgramService);
        }

        /// <summary>
        /// Returns true if EdFiProgramService instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiProgramService to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiProgramService input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.ServiceDescriptor == input.ServiceDescriptor ||
                    (this.ServiceDescriptor != null &&
                    this.ServiceDescriptor.Equals(input.ServiceDescriptor))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.ServiceDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ServiceDescriptor.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // ServiceDescriptor (string) maxLength
            if (this.ServiceDescriptor != null && this.ServiceDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ServiceDescriptor, length must be less than 306.", new [] { "ServiceDescriptor" });
            }

            yield break;
        }
    }

}
