/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiStudentAssessmentItem
    /// </summary>
    [DataContract(Name = "edFi_studentAssessmentItem")]
    public partial class EdFiStudentAssessmentItem : IEquatable<EdFiStudentAssessmentItem>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAssessmentItem" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiStudentAssessmentItem() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAssessmentItem" /> class.
        /// </summary>
        /// <param name="assessmentItemResultDescriptor">The analyzed result of a student&#39;s response to an assessment item. (required).</param>
        /// <param name="responseIndicatorDescriptor">Indicator of the response..</param>
        /// <param name="assessmentResponse">A student&#39;s response to a stimulus on a test..</param>
        /// <param name="descriptiveFeedback">The formative descriptive feedback that was given to a student in response to the results from a scored/evaluated assessment item..</param>
        /// <param name="itemNumber">The test question number for this student&#39;s test item..</param>
        /// <param name="rawScoreResult">A meaningful raw score of the performance of a student on an assessment item..</param>
        /// <param name="timeAssessed">The overall time a student actually spent during the assessment item..</param>
        /// <param name="assessmentItemReference">assessmentItemReference (required).</param>
        public EdFiStudentAssessmentItem(string assessmentItemResultDescriptor = default(string), string responseIndicatorDescriptor = default(string), string assessmentResponse = default(string), string descriptiveFeedback = default(string), int? itemNumber = default(int?), double? rawScoreResult = default(double?), string timeAssessed = default(string), EdFiAssessmentItemReference assessmentItemReference = default(EdFiAssessmentItemReference))
        {
            // to ensure "assessmentItemResultDescriptor" is required (not null)
            if (assessmentItemResultDescriptor == null)
            {
                throw new ArgumentNullException("assessmentItemResultDescriptor is a required property for EdFiStudentAssessmentItem and cannot be null");
            }
            this.AssessmentItemResultDescriptor = assessmentItemResultDescriptor;
            // to ensure "assessmentItemReference" is required (not null)
            if (assessmentItemReference == null)
            {
                throw new ArgumentNullException("assessmentItemReference is a required property for EdFiStudentAssessmentItem and cannot be null");
            }
            this.AssessmentItemReference = assessmentItemReference;
            this.ResponseIndicatorDescriptor = responseIndicatorDescriptor;
            this.AssessmentResponse = assessmentResponse;
            this.DescriptiveFeedback = descriptiveFeedback;
            this.ItemNumber = itemNumber;
            this.RawScoreResult = rawScoreResult;
            this.TimeAssessed = timeAssessed;
        }

        /// <summary>
        /// The analyzed result of a student&#39;s response to an assessment item.
        /// </summary>
        /// <value>The analyzed result of a student&#39;s response to an assessment item.</value>
        [DataMember(Name = "assessmentItemResultDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string AssessmentItemResultDescriptor { get; set; }

        /// <summary>
        /// Indicator of the response.
        /// </summary>
        /// <value>Indicator of the response.</value>
        [DataMember(Name = "responseIndicatorDescriptor", EmitDefaultValue = true)]
        public string ResponseIndicatorDescriptor { get; set; }

        /// <summary>
        /// A student&#39;s response to a stimulus on a test.
        /// </summary>
        /// <value>A student&#39;s response to a stimulus on a test.</value>
        [DataMember(Name = "assessmentResponse", EmitDefaultValue = true)]
        public string AssessmentResponse { get; set; }

        /// <summary>
        /// The formative descriptive feedback that was given to a student in response to the results from a scored/evaluated assessment item.
        /// </summary>
        /// <value>The formative descriptive feedback that was given to a student in response to the results from a scored/evaluated assessment item.</value>
        [DataMember(Name = "descriptiveFeedback", EmitDefaultValue = true)]
        public string DescriptiveFeedback { get; set; }

        /// <summary>
        /// The test question number for this student&#39;s test item.
        /// </summary>
        /// <value>The test question number for this student&#39;s test item.</value>
        [DataMember(Name = "itemNumber", EmitDefaultValue = true)]
        public int? ItemNumber { get; set; }

        /// <summary>
        /// A meaningful raw score of the performance of a student on an assessment item.
        /// </summary>
        /// <value>A meaningful raw score of the performance of a student on an assessment item.</value>
        [DataMember(Name = "rawScoreResult", EmitDefaultValue = true)]
        public double? RawScoreResult { get; set; }

        /// <summary>
        /// The overall time a student actually spent during the assessment item.
        /// </summary>
        /// <value>The overall time a student actually spent during the assessment item.</value>
        [DataMember(Name = "timeAssessed", EmitDefaultValue = true)]
        public string TimeAssessed { get; set; }

        /// <summary>
        /// Gets or Sets AssessmentItemReference
        /// </summary>
        [DataMember(Name = "assessmentItemReference", IsRequired = true, EmitDefaultValue = true)]
        public EdFiAssessmentItemReference AssessmentItemReference { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiStudentAssessmentItem {\n");
            sb.Append("  AssessmentItemResultDescriptor: ").Append(AssessmentItemResultDescriptor).Append("\n");
            sb.Append("  ResponseIndicatorDescriptor: ").Append(ResponseIndicatorDescriptor).Append("\n");
            sb.Append("  AssessmentResponse: ").Append(AssessmentResponse).Append("\n");
            sb.Append("  DescriptiveFeedback: ").Append(DescriptiveFeedback).Append("\n");
            sb.Append("  ItemNumber: ").Append(ItemNumber).Append("\n");
            sb.Append("  RawScoreResult: ").Append(RawScoreResult).Append("\n");
            sb.Append("  TimeAssessed: ").Append(TimeAssessed).Append("\n");
            sb.Append("  AssessmentItemReference: ").Append(AssessmentItemReference).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiStudentAssessmentItem);
        }

        /// <summary>
        /// Returns true if EdFiStudentAssessmentItem instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiStudentAssessmentItem to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiStudentAssessmentItem input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.AssessmentItemResultDescriptor == input.AssessmentItemResultDescriptor ||
                    (this.AssessmentItemResultDescriptor != null &&
                    this.AssessmentItemResultDescriptor.Equals(input.AssessmentItemResultDescriptor))
                ) && 
                (
                    this.ResponseIndicatorDescriptor == input.ResponseIndicatorDescriptor ||
                    (this.ResponseIndicatorDescriptor != null &&
                    this.ResponseIndicatorDescriptor.Equals(input.ResponseIndicatorDescriptor))
                ) && 
                (
                    this.AssessmentResponse == input.AssessmentResponse ||
                    (this.AssessmentResponse != null &&
                    this.AssessmentResponse.Equals(input.AssessmentResponse))
                ) && 
                (
                    this.DescriptiveFeedback == input.DescriptiveFeedback ||
                    (this.DescriptiveFeedback != null &&
                    this.DescriptiveFeedback.Equals(input.DescriptiveFeedback))
                ) && 
                (
                    this.ItemNumber == input.ItemNumber ||
                    (this.ItemNumber != null &&
                    this.ItemNumber.Equals(input.ItemNumber))
                ) && 
                (
                    this.RawScoreResult == input.RawScoreResult ||
                    (this.RawScoreResult != null &&
                    this.RawScoreResult.Equals(input.RawScoreResult))
                ) && 
                (
                    this.TimeAssessed == input.TimeAssessed ||
                    (this.TimeAssessed != null &&
                    this.TimeAssessed.Equals(input.TimeAssessed))
                ) && 
                (
                    this.AssessmentItemReference == input.AssessmentItemReference ||
                    (this.AssessmentItemReference != null &&
                    this.AssessmentItemReference.Equals(input.AssessmentItemReference))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.AssessmentItemResultDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.AssessmentItemResultDescriptor.GetHashCode();
                }
                if (this.ResponseIndicatorDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ResponseIndicatorDescriptor.GetHashCode();
                }
                if (this.AssessmentResponse != null)
                {
                    hashCode = (hashCode * 59) + this.AssessmentResponse.GetHashCode();
                }
                if (this.DescriptiveFeedback != null)
                {
                    hashCode = (hashCode * 59) + this.DescriptiveFeedback.GetHashCode();
                }
                if (this.ItemNumber != null)
                {
                    hashCode = (hashCode * 59) + this.ItemNumber.GetHashCode();
                }
                if (this.RawScoreResult != null)
                {
                    hashCode = (hashCode * 59) + this.RawScoreResult.GetHashCode();
                }
                if (this.TimeAssessed != null)
                {
                    hashCode = (hashCode * 59) + this.TimeAssessed.GetHashCode();
                }
                if (this.AssessmentItemReference != null)
                {
                    hashCode = (hashCode * 59) + this.AssessmentItemReference.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // AssessmentItemResultDescriptor (string) maxLength
            if (this.AssessmentItemResultDescriptor != null && this.AssessmentItemResultDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AssessmentItemResultDescriptor, length must be less than 306.", new [] { "AssessmentItemResultDescriptor" });
            }

            // ResponseIndicatorDescriptor (string) maxLength
            if (this.ResponseIndicatorDescriptor != null && this.ResponseIndicatorDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ResponseIndicatorDescriptor, length must be less than 306.", new [] { "ResponseIndicatorDescriptor" });
            }

            // AssessmentResponse (string) maxLength
            if (this.AssessmentResponse != null && this.AssessmentResponse.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AssessmentResponse, length must be less than 255.", new [] { "AssessmentResponse" });
            }

            // DescriptiveFeedback (string) maxLength
            if (this.DescriptiveFeedback != null && this.DescriptiveFeedback.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for DescriptiveFeedback, length must be less than 1024.", new [] { "DescriptiveFeedback" });
            }

            // TimeAssessed (string) maxLength
            if (this.TimeAssessed != null && this.TimeAssessed.Length > 30)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for TimeAssessed, length must be less than 30.", new [] { "TimeAssessed" });
            }

            yield break;
        }
    }

}
