/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiStudentAssessmentPeriod
    /// </summary>
    [DataContract(Name = "edFi_studentAssessmentPeriod")]
    public partial class EdFiStudentAssessmentPeriod : IEquatable<EdFiStudentAssessmentPeriod>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAssessmentPeriod" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiStudentAssessmentPeriod() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentAssessmentPeriod" /> class.
        /// </summary>
        /// <param name="assessmentPeriodDescriptor">The period of time in which an assessment is supposed to be administered (e.g., Beginning of Year, Middle of Year, End of Year). (required).</param>
        /// <param name="beginDate">The first date the assessment is to be administered..</param>
        /// <param name="endDate">The last date the assessment is to be administered..</param>
        public EdFiStudentAssessmentPeriod(string assessmentPeriodDescriptor = default(string), DateTime? beginDate = default(DateTime?), DateTime? endDate = default(DateTime?))
        {
            // to ensure "assessmentPeriodDescriptor" is required (not null)
            if (assessmentPeriodDescriptor == null)
            {
                throw new ArgumentNullException("assessmentPeriodDescriptor is a required property for EdFiStudentAssessmentPeriod and cannot be null");
            }
            this.AssessmentPeriodDescriptor = assessmentPeriodDescriptor;
            this.BeginDate = beginDate;
            this.EndDate = endDate;
        }

        /// <summary>
        /// The period of time in which an assessment is supposed to be administered (e.g., Beginning of Year, Middle of Year, End of Year).
        /// </summary>
        /// <value>The period of time in which an assessment is supposed to be administered (e.g., Beginning of Year, Middle of Year, End of Year).</value>
        [DataMember(Name = "assessmentPeriodDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string AssessmentPeriodDescriptor { get; set; }

        /// <summary>
        /// The first date the assessment is to be administered.
        /// </summary>
        /// <value>The first date the assessment is to be administered.</value>
        [DataMember(Name = "beginDate", EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime? BeginDate { get; set; }

        /// <summary>
        /// The last date the assessment is to be administered.
        /// </summary>
        /// <value>The last date the assessment is to be administered.</value>
        [DataMember(Name = "endDate", EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime? EndDate { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiStudentAssessmentPeriod {\n");
            sb.Append("  AssessmentPeriodDescriptor: ").Append(AssessmentPeriodDescriptor).Append("\n");
            sb.Append("  BeginDate: ").Append(BeginDate).Append("\n");
            sb.Append("  EndDate: ").Append(EndDate).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiStudentAssessmentPeriod);
        }

        /// <summary>
        /// Returns true if EdFiStudentAssessmentPeriod instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiStudentAssessmentPeriod to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiStudentAssessmentPeriod input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.AssessmentPeriodDescriptor == input.AssessmentPeriodDescriptor ||
                    (this.AssessmentPeriodDescriptor != null &&
                    this.AssessmentPeriodDescriptor.Equals(input.AssessmentPeriodDescriptor))
                ) && 
                (
                    this.BeginDate == input.BeginDate ||
                    (this.BeginDate != null &&
                    this.BeginDate.Equals(input.BeginDate))
                ) && 
                (
                    this.EndDate == input.EndDate ||
                    (this.EndDate != null &&
                    this.EndDate.Equals(input.EndDate))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.AssessmentPeriodDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.AssessmentPeriodDescriptor.GetHashCode();
                }
                if (this.BeginDate != null)
                {
                    hashCode = (hashCode * 59) + this.BeginDate.GetHashCode();
                }
                if (this.EndDate != null)
                {
                    hashCode = (hashCode * 59) + this.EndDate.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // AssessmentPeriodDescriptor (string) maxLength
            if (this.AssessmentPeriodDescriptor != null && this.AssessmentPeriodDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for AssessmentPeriodDescriptor, length must be less than 306.", new [] { "AssessmentPeriodDescriptor" });
            }

            yield break;
        }
    }

}
