/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiStudentCTEProgramAssociationCTEProgramService
    /// </summary>
    [DataContract(Name = "edFi_studentCTEProgramAssociationCTEProgramService")]
    public partial class EdFiStudentCTEProgramAssociationCTEProgramService : IEquatable<EdFiStudentCTEProgramAssociationCTEProgramService>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentCTEProgramAssociationCTEProgramService" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiStudentCTEProgramAssociationCTEProgramService() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiStudentCTEProgramAssociationCTEProgramService" /> class.
        /// </summary>
        /// <param name="cteProgramServiceDescriptor">Indicates the service being provided to the student by the CTE program. (required).</param>
        /// <param name="cipCode">Number and description of the CIP code associated with the student&#39;s CTE program..</param>
        /// <param name="primaryIndicator">True if service is a primary service..</param>
        /// <param name="serviceBeginDate">First date the student was in this option for the current school year..</param>
        /// <param name="serviceEndDate">Last date the student was in this option for the current school year..</param>
        public EdFiStudentCTEProgramAssociationCTEProgramService(string cteProgramServiceDescriptor = default(string), string cipCode = default(string), bool? primaryIndicator = default(bool?), DateTime? serviceBeginDate = default(DateTime?), DateTime? serviceEndDate = default(DateTime?))
        {
            // to ensure "cteProgramServiceDescriptor" is required (not null)
            if (cteProgramServiceDescriptor == null)
            {
                throw new ArgumentNullException("cteProgramServiceDescriptor is a required property for EdFiStudentCTEProgramAssociationCTEProgramService and cannot be null");
            }
            this.CteProgramServiceDescriptor = cteProgramServiceDescriptor;
            this.CipCode = cipCode;
            this.PrimaryIndicator = primaryIndicator;
            this.ServiceBeginDate = serviceBeginDate;
            this.ServiceEndDate = serviceEndDate;
        }

        /// <summary>
        /// Indicates the service being provided to the student by the CTE program.
        /// </summary>
        /// <value>Indicates the service being provided to the student by the CTE program.</value>
        [DataMember(Name = "cteProgramServiceDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string CteProgramServiceDescriptor { get; set; }

        /// <summary>
        /// Number and description of the CIP code associated with the student&#39;s CTE program.
        /// </summary>
        /// <value>Number and description of the CIP code associated with the student&#39;s CTE program.</value>
        [DataMember(Name = "cipCode", EmitDefaultValue = true)]
        public string CipCode { get; set; }

        /// <summary>
        /// True if service is a primary service.
        /// </summary>
        /// <value>True if service is a primary service.</value>
        [DataMember(Name = "primaryIndicator", EmitDefaultValue = true)]
        public bool? PrimaryIndicator { get; set; }

        /// <summary>
        /// First date the student was in this option for the current school year.
        /// </summary>
        /// <value>First date the student was in this option for the current school year.</value>
        [DataMember(Name = "serviceBeginDate", EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime? ServiceBeginDate { get; set; }

        /// <summary>
        /// Last date the student was in this option for the current school year.
        /// </summary>
        /// <value>Last date the student was in this option for the current school year.</value>
        [DataMember(Name = "serviceEndDate", EmitDefaultValue = true)]
        [JsonConverter(typeof(OpenAPIDateConverter))]
        public DateTime? ServiceEndDate { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiStudentCTEProgramAssociationCTEProgramService {\n");
            sb.Append("  CteProgramServiceDescriptor: ").Append(CteProgramServiceDescriptor).Append("\n");
            sb.Append("  CipCode: ").Append(CipCode).Append("\n");
            sb.Append("  PrimaryIndicator: ").Append(PrimaryIndicator).Append("\n");
            sb.Append("  ServiceBeginDate: ").Append(ServiceBeginDate).Append("\n");
            sb.Append("  ServiceEndDate: ").Append(ServiceEndDate).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiStudentCTEProgramAssociationCTEProgramService);
        }

        /// <summary>
        /// Returns true if EdFiStudentCTEProgramAssociationCTEProgramService instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiStudentCTEProgramAssociationCTEProgramService to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiStudentCTEProgramAssociationCTEProgramService input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.CteProgramServiceDescriptor == input.CteProgramServiceDescriptor ||
                    (this.CteProgramServiceDescriptor != null &&
                    this.CteProgramServiceDescriptor.Equals(input.CteProgramServiceDescriptor))
                ) && 
                (
                    this.CipCode == input.CipCode ||
                    (this.CipCode != null &&
                    this.CipCode.Equals(input.CipCode))
                ) && 
                (
                    this.PrimaryIndicator == input.PrimaryIndicator ||
                    (this.PrimaryIndicator != null &&
                    this.PrimaryIndicator.Equals(input.PrimaryIndicator))
                ) && 
                (
                    this.ServiceBeginDate == input.ServiceBeginDate ||
                    (this.ServiceBeginDate != null &&
                    this.ServiceBeginDate.Equals(input.ServiceBeginDate))
                ) && 
                (
                    this.ServiceEndDate == input.ServiceEndDate ||
                    (this.ServiceEndDate != null &&
                    this.ServiceEndDate.Equals(input.ServiceEndDate))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.CteProgramServiceDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.CteProgramServiceDescriptor.GetHashCode();
                }
                if (this.CipCode != null)
                {
                    hashCode = (hashCode * 59) + this.CipCode.GetHashCode();
                }
                if (this.PrimaryIndicator != null)
                {
                    hashCode = (hashCode * 59) + this.PrimaryIndicator.GetHashCode();
                }
                if (this.ServiceBeginDate != null)
                {
                    hashCode = (hashCode * 59) + this.ServiceBeginDate.GetHashCode();
                }
                if (this.ServiceEndDate != null)
                {
                    hashCode = (hashCode * 59) + this.ServiceEndDate.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // CteProgramServiceDescriptor (string) maxLength
            if (this.CteProgramServiceDescriptor != null && this.CteProgramServiceDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CteProgramServiceDescriptor, length must be less than 306.", new [] { "CteProgramServiceDescriptor" });
            }

            // CipCode (string) maxLength
            if (this.CipCode != null && this.CipCode.Length > 120)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CipCode, length must be less than 120.", new [] { "CipCode" });
            }

            yield break;
        }
    }

}
