/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiSurveyQuestionMatrix
    /// </summary>
    [DataContract(Name = "edFi_surveyQuestionMatrix")]
    public partial class EdFiSurveyQuestionMatrix : IEquatable<EdFiSurveyQuestionMatrix>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyQuestionMatrix" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiSurveyQuestionMatrix() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiSurveyQuestionMatrix" /> class.
        /// </summary>
        /// <param name="matrixElement">For matrix questions, the text identifying each row of the matrix. (required).</param>
        /// <param name="maxRawScore">The maximum score possible on a survey..</param>
        /// <param name="minRawScore">The minimum score possible on a survey..</param>
        public EdFiSurveyQuestionMatrix(string matrixElement = default(string), int? maxRawScore = default(int?), int? minRawScore = default(int?))
        {
            // to ensure "matrixElement" is required (not null)
            if (matrixElement == null)
            {
                throw new ArgumentNullException("matrixElement is a required property for EdFiSurveyQuestionMatrix and cannot be null");
            }
            this.MatrixElement = matrixElement;
            this.MaxRawScore = maxRawScore;
            this.MinRawScore = minRawScore;
        }

        /// <summary>
        /// For matrix questions, the text identifying each row of the matrix.
        /// </summary>
        /// <value>For matrix questions, the text identifying each row of the matrix.</value>
        [DataMember(Name = "matrixElement", IsRequired = true, EmitDefaultValue = true)]
        public string MatrixElement { get; set; }

        /// <summary>
        /// The maximum score possible on a survey.
        /// </summary>
        /// <value>The maximum score possible on a survey.</value>
        [DataMember(Name = "maxRawScore", EmitDefaultValue = true)]
        public int? MaxRawScore { get; set; }

        /// <summary>
        /// The minimum score possible on a survey.
        /// </summary>
        /// <value>The minimum score possible on a survey.</value>
        [DataMember(Name = "minRawScore", EmitDefaultValue = true)]
        public int? MinRawScore { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class EdFiSurveyQuestionMatrix {\n");
            sb.Append("  MatrixElement: ").Append(MatrixElement).Append("\n");
            sb.Append("  MaxRawScore: ").Append(MaxRawScore).Append("\n");
            sb.Append("  MinRawScore: ").Append(MinRawScore).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiSurveyQuestionMatrix);
        }

        /// <summary>
        /// Returns true if EdFiSurveyQuestionMatrix instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiSurveyQuestionMatrix to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiSurveyQuestionMatrix input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.MatrixElement == input.MatrixElement ||
                    (this.MatrixElement != null &&
                    this.MatrixElement.Equals(input.MatrixElement))
                ) && 
                (
                    this.MaxRawScore == input.MaxRawScore ||
                    (this.MaxRawScore != null &&
                    this.MaxRawScore.Equals(input.MaxRawScore))
                ) && 
                (
                    this.MinRawScore == input.MinRawScore ||
                    (this.MinRawScore != null &&
                    this.MinRawScore.Equals(input.MinRawScore))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.MatrixElement != null)
                {
                    hashCode = (hashCode * 59) + this.MatrixElement.GetHashCode();
                }
                if (this.MaxRawScore != null)
                {
                    hashCode = (hashCode * 59) + this.MaxRawScore.GetHashCode();
                }
                if (this.MinRawScore != null)
                {
                    hashCode = (hashCode * 59) + this.MinRawScore.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // MatrixElement (string) maxLength
            if (this.MatrixElement != null && this.MatrixElement.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for MatrixElement, length must be less than 255.", new [] { "MatrixElement" });
            }

            yield break;
        }
    }

}
