/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// TrackedChangesEdFiCommunityProviderLicenseKey
    /// </summary>
    [DataContract(Name = "trackedChanges_edFi_communityProviderLicenseKey")]
    public partial class TrackedChangesEdFiCommunityProviderLicenseKey : IEquatable<TrackedChangesEdFiCommunityProviderLicenseKey>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TrackedChangesEdFiCommunityProviderLicenseKey" /> class.
        /// </summary>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization..</param>
        /// <param name="licensingOrganization">The organization issuing the license..</param>
        /// <param name="communityProviderId">The identifier assigned to a community provider..</param>
        public TrackedChangesEdFiCommunityProviderLicenseKey(string licenseIdentifier = default(string), string licensingOrganization = default(string), int communityProviderId = default(int))
        {
            this.LicenseIdentifier = licenseIdentifier;
            this.LicensingOrganization = licensingOrganization;
            this.CommunityProviderId = communityProviderId;
        }

        /// <summary>
        /// The unique identifier issued by the licensing organization.
        /// </summary>
        /// <value>The unique identifier issued by the licensing organization.</value>
        [DataMember(Name = "licenseIdentifier", EmitDefaultValue = false)]
        public string LicenseIdentifier { get; set; }

        /// <summary>
        /// The organization issuing the license.
        /// </summary>
        /// <value>The organization issuing the license.</value>
        [DataMember(Name = "licensingOrganization", EmitDefaultValue = false)]
        public string LicensingOrganization { get; set; }

        /// <summary>
        /// The identifier assigned to a community provider.
        /// </summary>
        /// <value>The identifier assigned to a community provider.</value>
        [DataMember(Name = "communityProviderId", EmitDefaultValue = false)]
        public int CommunityProviderId { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class TrackedChangesEdFiCommunityProviderLicenseKey {\n");
            sb.Append("  LicenseIdentifier: ").Append(LicenseIdentifier).Append("\n");
            sb.Append("  LicensingOrganization: ").Append(LicensingOrganization).Append("\n");
            sb.Append("  CommunityProviderId: ").Append(CommunityProviderId).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as TrackedChangesEdFiCommunityProviderLicenseKey);
        }

        /// <summary>
        /// Returns true if TrackedChangesEdFiCommunityProviderLicenseKey instances are equal
        /// </summary>
        /// <param name="input">Instance of TrackedChangesEdFiCommunityProviderLicenseKey to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(TrackedChangesEdFiCommunityProviderLicenseKey input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.LicenseIdentifier == input.LicenseIdentifier ||
                    (this.LicenseIdentifier != null &&
                    this.LicenseIdentifier.Equals(input.LicenseIdentifier))
                ) && 
                (
                    this.LicensingOrganization == input.LicensingOrganization ||
                    (this.LicensingOrganization != null &&
                    this.LicensingOrganization.Equals(input.LicensingOrganization))
                ) && 
                (
                    this.CommunityProviderId == input.CommunityProviderId ||
                    this.CommunityProviderId.Equals(input.CommunityProviderId)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.LicenseIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.LicenseIdentifier.GetHashCode();
                }
                if (this.LicensingOrganization != null)
                {
                    hashCode = (hashCode * 59) + this.LicensingOrganization.GetHashCode();
                }
                hashCode = (hashCode * 59) + this.CommunityProviderId.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // LicenseIdentifier (string) maxLength
            if (this.LicenseIdentifier != null && this.LicenseIdentifier.Length > 20)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for LicenseIdentifier, length must be less than 20.", new [] { "LicenseIdentifier" });
            }

            // LicensingOrganization (string) maxLength
            if (this.LicensingOrganization != null && this.LicensingOrganization.Length > 75)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for LicensingOrganization, length must be less than 75.", new [] { "LicensingOrganization" });
            }

            yield break;
        }
    }

}
