/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// WiDisciplineActionExtension
    /// </summary>
    [DataContract(Name = "wi_disciplineActionExtension")]
    public partial class WiDisciplineActionExtension : IEquatable<WiDisciplineActionExtension>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WiDisciplineActionExtension" /> class.
        /// </summary>
        /// <param name="modifiedTermDescriptor">Identifies if the duration of the expulsion has been modified from the federally required one year of expulsion for a fire arms incident..</param>
        /// <param name="earlyReinstatementCondition">Following an expulsion, the student had early reinstatement condition to return to school prior to the end of the expulsion..</param>
        public WiDisciplineActionExtension(string modifiedTermDescriptor = default(string), bool? earlyReinstatementCondition = default(bool?))
        {
            this.ModifiedTermDescriptor = modifiedTermDescriptor;
            this.EarlyReinstatementCondition = earlyReinstatementCondition;
        }

        /// <summary>
        /// Identifies if the duration of the expulsion has been modified from the federally required one year of expulsion for a fire arms incident.
        /// </summary>
        /// <value>Identifies if the duration of the expulsion has been modified from the federally required one year of expulsion for a fire arms incident.</value>
        [DataMember(Name = "modifiedTermDescriptor", EmitDefaultValue = true)]
        public string ModifiedTermDescriptor { get; set; }

        /// <summary>
        /// Following an expulsion, the student had early reinstatement condition to return to school prior to the end of the expulsion.
        /// </summary>
        /// <value>Following an expulsion, the student had early reinstatement condition to return to school prior to the end of the expulsion.</value>
        [DataMember(Name = "earlyReinstatementCondition", EmitDefaultValue = true)]
        public bool? EarlyReinstatementCondition { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class WiDisciplineActionExtension {\n");
            sb.Append("  ModifiedTermDescriptor: ").Append(ModifiedTermDescriptor).Append("\n");
            sb.Append("  EarlyReinstatementCondition: ").Append(EarlyReinstatementCondition).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as WiDisciplineActionExtension);
        }

        /// <summary>
        /// Returns true if WiDisciplineActionExtension instances are equal
        /// </summary>
        /// <param name="input">Instance of WiDisciplineActionExtension to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(WiDisciplineActionExtension input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.ModifiedTermDescriptor == input.ModifiedTermDescriptor ||
                    (this.ModifiedTermDescriptor != null &&
                    this.ModifiedTermDescriptor.Equals(input.ModifiedTermDescriptor))
                ) && 
                (
                    this.EarlyReinstatementCondition == input.EarlyReinstatementCondition ||
                    (this.EarlyReinstatementCondition != null &&
                    this.EarlyReinstatementCondition.Equals(input.EarlyReinstatementCondition))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.ModifiedTermDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ModifiedTermDescriptor.GetHashCode();
                }
                if (this.EarlyReinstatementCondition != null)
                {
                    hashCode = (hashCode * 59) + this.EarlyReinstatementCondition.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // ModifiedTermDescriptor (string) maxLength
            if (this.ModifiedTermDescriptor != null && this.ModifiedTermDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ModifiedTermDescriptor, length must be less than 306.", new [] { "ModifiedTermDescriptor" });
            }

            yield break;
        }
    }

}
