/*
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * The version of the OpenAPI document: 3
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// WiStudentSchoolAssociationReceivingService
    /// </summary>
    [DataContract(Name = "wi_studentSchoolAssociationReceivingService")]
    public partial class WiStudentSchoolAssociationReceivingService : IEquatable<WiStudentSchoolAssociationReceivingService>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WiStudentSchoolAssociationReceivingService" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected WiStudentSchoolAssociationReceivingService() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="WiStudentSchoolAssociationReceivingService" /> class.
        /// </summary>
        /// <param name="countDateNameDescriptor">The count date name for the student status (e.g 3rd Fri Sept) (required).</param>
        /// <param name="countDateReceivingServiceDescriptor">The status of the student during the specific count date (e.g. Present) (required).</param>
        public WiStudentSchoolAssociationReceivingService(string countDateNameDescriptor = default(string), string countDateReceivingServiceDescriptor = default(string))
        {
            // to ensure "countDateNameDescriptor" is required (not null)
            if (countDateNameDescriptor == null)
            {
                throw new ArgumentNullException("countDateNameDescriptor is a required property for WiStudentSchoolAssociationReceivingService and cannot be null");
            }
            this.CountDateNameDescriptor = countDateNameDescriptor;
            // to ensure "countDateReceivingServiceDescriptor" is required (not null)
            if (countDateReceivingServiceDescriptor == null)
            {
                throw new ArgumentNullException("countDateReceivingServiceDescriptor is a required property for WiStudentSchoolAssociationReceivingService and cannot be null");
            }
            this.CountDateReceivingServiceDescriptor = countDateReceivingServiceDescriptor;
        }

        /// <summary>
        /// The count date name for the student status (e.g 3rd Fri Sept)
        /// </summary>
        /// <value>The count date name for the student status (e.g 3rd Fri Sept)</value>
        [DataMember(Name = "countDateNameDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string CountDateNameDescriptor { get; set; }

        /// <summary>
        /// The status of the student during the specific count date (e.g. Present)
        /// </summary>
        /// <value>The status of the student during the specific count date (e.g. Present)</value>
        [DataMember(Name = "countDateReceivingServiceDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string CountDateReceivingServiceDescriptor { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class WiStudentSchoolAssociationReceivingService {\n");
            sb.Append("  CountDateNameDescriptor: ").Append(CountDateNameDescriptor).Append("\n");
            sb.Append("  CountDateReceivingServiceDescriptor: ").Append(CountDateReceivingServiceDescriptor).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as WiStudentSchoolAssociationReceivingService);
        }

        /// <summary>
        /// Returns true if WiStudentSchoolAssociationReceivingService instances are equal
        /// </summary>
        /// <param name="input">Instance of WiStudentSchoolAssociationReceivingService to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(WiStudentSchoolAssociationReceivingService input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.CountDateNameDescriptor == input.CountDateNameDescriptor ||
                    (this.CountDateNameDescriptor != null &&
                    this.CountDateNameDescriptor.Equals(input.CountDateNameDescriptor))
                ) && 
                (
                    this.CountDateReceivingServiceDescriptor == input.CountDateReceivingServiceDescriptor ||
                    (this.CountDateReceivingServiceDescriptor != null &&
                    this.CountDateReceivingServiceDescriptor.Equals(input.CountDateReceivingServiceDescriptor))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.CountDateNameDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.CountDateNameDescriptor.GetHashCode();
                }
                if (this.CountDateReceivingServiceDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.CountDateReceivingServiceDescriptor.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // CountDateNameDescriptor (string) maxLength
            if (this.CountDateNameDescriptor != null && this.CountDateNameDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CountDateNameDescriptor, length must be less than 306.", new [] { "CountDateNameDescriptor" });
            }

            // CountDateReceivingServiceDescriptor (string) maxLength
            if (this.CountDateReceivingServiceDescriptor != null && this.CountDateReceivingServiceDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for CountDateReceivingServiceDescriptor, length must be less than 306.", new [] { "CountDateReceivingServiceDescriptor" });
            }

            yield break;
        }
    }

}
