/*
 * Validations API Endpoint
 *
 * L2 data validation from WI DPI.
 *
 * The version of the OpenAPI document: 1.0.0
 * Generated by: https://github.com/openapitools/openapi-generator.git
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using System.ComponentModel.DataAnnotations;
using OpenAPIDateConverter = EdFi.OdsApi.Sdk.Client.OpenAPIDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Validations
{
    /// <summary>
    /// Logic which identifies errors within the data or which require further investigation.
    /// </summary>
    [DataContract(Name = "ValidationRule")]
    public partial class ValidationRule : IEquatable<ValidationRule>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ValidationRule" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected ValidationRule() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="ValidationRule" /> class.
        /// </summary>
        /// <param name="id">id.</param>
        /// <param name="ruleIdentifier">This is the unique Id for a validation rule. (required).</param>
        /// <param name="ruleSource">The source or origin of the rule. (required).</param>
        /// <param name="helpUrl">A link to more information about the rule and how to resolve it..</param>
        /// <param name="shortDescription">This is non-structured ASCII text that will include the short details that were used in the evaluation of the validation rule..</param>
        /// <param name="description">This is non-structured ASCII text that will include the details that were used in the evaluation of the validation rule. (required).</param>
        /// <param name="ruleStatusDescriptor">The current status of if the rule. Examples are &#39;Active&#39;, &#39;Under Analysis&#39;, &#39;Inactive&#39;, &#39;Deprecated&#39;. (required).</param>
        /// <param name="category">This is a category for the type of validation rule.  Examples might be &#39;Student Demographics&#39;, &#39;Special Education&#39;, or &#39;Attendance&#39;.</param>
        /// <param name="severityDescriptor">This specifies whether the validation rule is a &#39;Warning&#39;, &#39;Minor Validation Error&#39;, &#39;Major Validation Error&#39; or other value standardized by the API (required).</param>
        /// <param name="externalRuleId">Refers back to a unique identifier for this rule in another system (such as a state-maintained repository of validation rules).</param>
        /// <param name="validationLogicTypeDescriptor">Specifies the language that the validation logic is represented in, ie SQL or Pseudo-code.</param>
        /// <param name="validationLogic">Has the actual code or pseudo-code that is used to find validation errors..</param>
        public ValidationRule(string id = default(string), string ruleIdentifier = default(string), string ruleSource = default(string), string helpUrl = default(string), string shortDescription = default(string), string description = default(string), string ruleStatusDescriptor = default(string), string category = default(string), string severityDescriptor = default(string), string externalRuleId = default(string), string validationLogicTypeDescriptor = default(string), string validationLogic = default(string))
        {
            // to ensure "ruleIdentifier" is required (not null)
            if (ruleIdentifier == null)
            {
                throw new ArgumentNullException("ruleIdentifier is a required property for ValidationRule and cannot be null");
            }
            this.RuleIdentifier = ruleIdentifier;
            // to ensure "ruleSource" is required (not null)
            if (ruleSource == null)
            {
                throw new ArgumentNullException("ruleSource is a required property for ValidationRule and cannot be null");
            }
            this.RuleSource = ruleSource;
            // to ensure "description" is required (not null)
            if (description == null)
            {
                throw new ArgumentNullException("description is a required property for ValidationRule and cannot be null");
            }
            this.Description = description;
            // to ensure "ruleStatusDescriptor" is required (not null)
            if (ruleStatusDescriptor == null)
            {
                throw new ArgumentNullException("ruleStatusDescriptor is a required property for ValidationRule and cannot be null");
            }
            this.RuleStatusDescriptor = ruleStatusDescriptor;
            // to ensure "severityDescriptor" is required (not null)
            if (severityDescriptor == null)
            {
                throw new ArgumentNullException("severityDescriptor is a required property for ValidationRule and cannot be null");
            }
            this.SeverityDescriptor = severityDescriptor;
            this.Id = id;
            this.HelpUrl = helpUrl;
            this.ShortDescription = shortDescription;
            this.Category = category;
            this.ExternalRuleId = externalRuleId;
            this.ValidationLogicTypeDescriptor = validationLogicTypeDescriptor;
            this.ValidationLogic = validationLogic;
        }

        /// <summary>
        /// Gets or Sets Id
        /// </summary>
        [DataMember(Name = "id", EmitDefaultValue = false)]
        public string Id { get; set; }

        /// <summary>
        /// This is the unique Id for a validation rule.
        /// </summary>
        /// <value>This is the unique Id for a validation rule.</value>
        [DataMember(Name = "ruleIdentifier", IsRequired = true, EmitDefaultValue = true)]
        public string RuleIdentifier { get; set; }

        /// <summary>
        /// The source or origin of the rule.
        /// </summary>
        /// <value>The source or origin of the rule.</value>
        [DataMember(Name = "ruleSource", IsRequired = true, EmitDefaultValue = true)]
        public string RuleSource { get; set; }

        /// <summary>
        /// A link to more information about the rule and how to resolve it.
        /// </summary>
        /// <value>A link to more information about the rule and how to resolve it.</value>
        [DataMember(Name = "helpUrl", EmitDefaultValue = false)]
        public string HelpUrl { get; set; }

        /// <summary>
        /// This is non-structured ASCII text that will include the short details that were used in the evaluation of the validation rule.
        /// </summary>
        /// <value>This is non-structured ASCII text that will include the short details that were used in the evaluation of the validation rule.</value>
        [DataMember(Name = "shortDescription", EmitDefaultValue = false)]
        public string ShortDescription { get; set; }

        /// <summary>
        /// This is non-structured ASCII text that will include the details that were used in the evaluation of the validation rule.
        /// </summary>
        /// <value>This is non-structured ASCII text that will include the details that were used in the evaluation of the validation rule.</value>
        [DataMember(Name = "description", IsRequired = true, EmitDefaultValue = true)]
        public string Description { get; set; }

        /// <summary>
        /// The current status of if the rule. Examples are &#39;Active&#39;, &#39;Under Analysis&#39;, &#39;Inactive&#39;, &#39;Deprecated&#39;.
        /// </summary>
        /// <value>The current status of if the rule. Examples are &#39;Active&#39;, &#39;Under Analysis&#39;, &#39;Inactive&#39;, &#39;Deprecated&#39;.</value>
        [DataMember(Name = "ruleStatusDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string RuleStatusDescriptor { get; set; }

        /// <summary>
        /// This is a category for the type of validation rule.  Examples might be &#39;Student Demographics&#39;, &#39;Special Education&#39;, or &#39;Attendance&#39;
        /// </summary>
        /// <value>This is a category for the type of validation rule.  Examples might be &#39;Student Demographics&#39;, &#39;Special Education&#39;, or &#39;Attendance&#39;</value>
        [DataMember(Name = "category", EmitDefaultValue = false)]
        public string Category { get; set; }

        /// <summary>
        /// This specifies whether the validation rule is a &#39;Warning&#39;, &#39;Minor Validation Error&#39;, &#39;Major Validation Error&#39; or other value standardized by the API
        /// </summary>
        /// <value>This specifies whether the validation rule is a &#39;Warning&#39;, &#39;Minor Validation Error&#39;, &#39;Major Validation Error&#39; or other value standardized by the API</value>
        [DataMember(Name = "severityDescriptor", IsRequired = true, EmitDefaultValue = true)]
        public string SeverityDescriptor { get; set; }

        /// <summary>
        /// Refers back to a unique identifier for this rule in another system (such as a state-maintained repository of validation rules)
        /// </summary>
        /// <value>Refers back to a unique identifier for this rule in another system (such as a state-maintained repository of validation rules)</value>
        [DataMember(Name = "externalRuleId", EmitDefaultValue = false)]
        public string ExternalRuleId { get; set; }

        /// <summary>
        /// Specifies the language that the validation logic is represented in, ie SQL or Pseudo-code
        /// </summary>
        /// <value>Specifies the language that the validation logic is represented in, ie SQL or Pseudo-code</value>
        [DataMember(Name = "validationLogicTypeDescriptor", EmitDefaultValue = false)]
        public string ValidationLogicTypeDescriptor { get; set; }

        /// <summary>
        /// Has the actual code or pseudo-code that is used to find validation errors.
        /// </summary>
        /// <value>Has the actual code or pseudo-code that is used to find validation errors.</value>
        [DataMember(Name = "validationLogic", EmitDefaultValue = false)]
        public string ValidationLogic { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("class ValidationRule {\n");
            sb.Append("  Id: ").Append(Id).Append("\n");
            sb.Append("  RuleIdentifier: ").Append(RuleIdentifier).Append("\n");
            sb.Append("  RuleSource: ").Append(RuleSource).Append("\n");
            sb.Append("  HelpUrl: ").Append(HelpUrl).Append("\n");
            sb.Append("  ShortDescription: ").Append(ShortDescription).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  RuleStatusDescriptor: ").Append(RuleStatusDescriptor).Append("\n");
            sb.Append("  Category: ").Append(Category).Append("\n");
            sb.Append("  SeverityDescriptor: ").Append(SeverityDescriptor).Append("\n");
            sb.Append("  ExternalRuleId: ").Append(ExternalRuleId).Append("\n");
            sb.Append("  ValidationLogicTypeDescriptor: ").Append(ValidationLogicTypeDescriptor).Append("\n");
            sb.Append("  ValidationLogic: ").Append(ValidationLogic).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }

        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return Newtonsoft.Json.JsonConvert.SerializeObject(this, Newtonsoft.Json.Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ValidationRule);
        }

        /// <summary>
        /// Returns true if ValidationRule instances are equal
        /// </summary>
        /// <param name="input">Instance of ValidationRule to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ValidationRule input)
        {
            if (input == null)
            {
                return false;
            }
            return 
                (
                    this.Id == input.Id ||
                    (this.Id != null &&
                    this.Id.Equals(input.Id))
                ) && 
                (
                    this.RuleIdentifier == input.RuleIdentifier ||
                    (this.RuleIdentifier != null &&
                    this.RuleIdentifier.Equals(input.RuleIdentifier))
                ) && 
                (
                    this.RuleSource == input.RuleSource ||
                    (this.RuleSource != null &&
                    this.RuleSource.Equals(input.RuleSource))
                ) && 
                (
                    this.HelpUrl == input.HelpUrl ||
                    (this.HelpUrl != null &&
                    this.HelpUrl.Equals(input.HelpUrl))
                ) && 
                (
                    this.ShortDescription == input.ShortDescription ||
                    (this.ShortDescription != null &&
                    this.ShortDescription.Equals(input.ShortDescription))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.RuleStatusDescriptor == input.RuleStatusDescriptor ||
                    (this.RuleStatusDescriptor != null &&
                    this.RuleStatusDescriptor.Equals(input.RuleStatusDescriptor))
                ) && 
                (
                    this.Category == input.Category ||
                    (this.Category != null &&
                    this.Category.Equals(input.Category))
                ) && 
                (
                    this.SeverityDescriptor == input.SeverityDescriptor ||
                    (this.SeverityDescriptor != null &&
                    this.SeverityDescriptor.Equals(input.SeverityDescriptor))
                ) && 
                (
                    this.ExternalRuleId == input.ExternalRuleId ||
                    (this.ExternalRuleId != null &&
                    this.ExternalRuleId.Equals(input.ExternalRuleId))
                ) && 
                (
                    this.ValidationLogicTypeDescriptor == input.ValidationLogicTypeDescriptor ||
                    (this.ValidationLogicTypeDescriptor != null &&
                    this.ValidationLogicTypeDescriptor.Equals(input.ValidationLogicTypeDescriptor))
                ) && 
                (
                    this.ValidationLogic == input.ValidationLogic ||
                    (this.ValidationLogic != null &&
                    this.ValidationLogic.Equals(input.ValidationLogic))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Id != null)
                {
                    hashCode = (hashCode * 59) + this.Id.GetHashCode();
                }
                if (this.RuleIdentifier != null)
                {
                    hashCode = (hashCode * 59) + this.RuleIdentifier.GetHashCode();
                }
                if (this.RuleSource != null)
                {
                    hashCode = (hashCode * 59) + this.RuleSource.GetHashCode();
                }
                if (this.HelpUrl != null)
                {
                    hashCode = (hashCode * 59) + this.HelpUrl.GetHashCode();
                }
                if (this.ShortDescription != null)
                {
                    hashCode = (hashCode * 59) + this.ShortDescription.GetHashCode();
                }
                if (this.Description != null)
                {
                    hashCode = (hashCode * 59) + this.Description.GetHashCode();
                }
                if (this.RuleStatusDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.RuleStatusDescriptor.GetHashCode();
                }
                if (this.Category != null)
                {
                    hashCode = (hashCode * 59) + this.Category.GetHashCode();
                }
                if (this.SeverityDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.SeverityDescriptor.GetHashCode();
                }
                if (this.ExternalRuleId != null)
                {
                    hashCode = (hashCode * 59) + this.ExternalRuleId.GetHashCode();
                }
                if (this.ValidationLogicTypeDescriptor != null)
                {
                    hashCode = (hashCode * 59) + this.ValidationLogicTypeDescriptor.GetHashCode();
                }
                if (this.ValidationLogic != null)
                {
                    hashCode = (hashCode * 59) + this.ValidationLogic.GetHashCode();
                }
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        public IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> Validate(ValidationContext validationContext)
        {
            // RuleIdentifier (string) maxLength
            if (this.RuleIdentifier != null && this.RuleIdentifier.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for RuleIdentifier, length must be less than 100.", new [] { "RuleIdentifier" });
            }

            // RuleSource (string) maxLength
            if (this.RuleSource != null && this.RuleSource.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for RuleSource, length must be less than 100.", new [] { "RuleSource" });
            }

            // HelpUrl (string) maxLength
            if (this.HelpUrl != null && this.HelpUrl.Length > 255)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for HelpUrl, length must be less than 255.", new [] { "HelpUrl" });
            }

            // ShortDescription (string) maxLength
            if (this.ShortDescription != null && this.ShortDescription.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ShortDescription, length must be less than 1024.", new [] { "ShortDescription" });
            }

            // Description (string) maxLength
            if (this.Description != null && this.Description.Length > 4000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Description, length must be less than 4000.", new [] { "Description" });
            }

            // RuleStatusDescriptor (string) maxLength
            if (this.RuleStatusDescriptor != null && this.RuleStatusDescriptor.Length > 50)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for RuleStatusDescriptor, length must be less than 50.", new [] { "RuleStatusDescriptor" });
            }

            // Category (string) maxLength
            if (this.Category != null && this.Category.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Category, length must be less than 1024.", new [] { "Category" });
            }

            // SeverityDescriptor (string) maxLength
            if (this.SeverityDescriptor != null && this.SeverityDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for SeverityDescriptor, length must be less than 306.", new [] { "SeverityDescriptor" });
            }

            // ExternalRuleId (string) maxLength
            if (this.ExternalRuleId != null && this.ExternalRuleId.Length > 100)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ExternalRuleId, length must be less than 100.", new [] { "ExternalRuleId" });
            }

            // ValidationLogicTypeDescriptor (string) maxLength
            if (this.ValidationLogicTypeDescriptor != null && this.ValidationLogicTypeDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ValidationLogicTypeDescriptor, length must be less than 306.", new [] { "ValidationLogicTypeDescriptor" });
            }

            // ValidationLogic (string) maxLength
            if (this.ValidationLogic != null && this.ValidationLogic.Length > 4000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for ValidationLogic, length must be less than 4000.", new [] { "ValidationLogic" });
            }

            yield break;
        }
    }

}
